from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant.system import HttpMethod


class GetExchangeCurrenciesService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/common/currencys"

        def parse(dict_data):
            data_list = dict_data.get("data", [])
            return data_list if len(data_list) else []

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET, channel, self.params, parse)










class SubuserTransferability:
    """
    The trade information with price and amount etc.

    :member
        subUid: sub user ID.
        userState: sub user account state, states see SubUidState.
    """

    def __init__(self):
        self.transferrable = ""
        self.accountType = ""
        self.subUid = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.subUid, format_data + "subUid")
        PrintBasic.print_basic(self.accountType, format_data + "accountType")
        PrintBasic.print_basic(self.transferrable, format_data + "transferrable")



from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant import *
from huobi.utils.json_parser import default_parse_fill_directly
from huobi.model.algo import *


class PostCancelOrderService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/algo-orders/cancellation"

        # {'code': 200, 'data': {'accepted': [], 'rejected': ['test001', 'test002']}}
        def parse(dict_data):
            data = dict_data.get("data", {})
            return default_parse_fill_directly(data, CancelOrderResult)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)



from huobi.client.market import MarketClient
from huobi.constant import *


def callback(mbp_event: 'MbpIncreaseEvent'):
    mbp_event.print_object()


def error(e: 'HuobiApiException'):
    print(e.error_code + e.error_message)


market_client = MarketClient(init_log=True)
market_client.req_mbp("btcusdt", MbpLevel.MBP5, callback, error)



from huobi.utils import *
from huobi.constant import *
from huobi.utils.input_checker import check_in_list


class SubuserClient(object):
    def __init__(self, **kwargs):
        """
        Create the request client instance.
        :param kwargs: The option of request connection.
            api_key: The public key applied from Huobi.
            secret_key: The private key applied from Huobi.
            url: The URL name like "https://api.huobi.pro".
            init_log: Init logger, default is False, True will init logger handler
        """
        self.__kwargs = kwargs

    def post_create_subuser(self, user_list):
        check_should_not_none(user_list, 'userList')

        params = user_list
        from huobi.service.subuser.post_create_subuser import PostSubuserCreationService
        return PostSubuserCreationService(params).request(**self.__kwargs)

    def post_set_tradable_market(self, sub_uids, account_type: 'SubuserTradePrivilegeType',
                                 activation: 'SubUserTradeStatus'):
        check_should_not_none(sub_uids, 'subUids')
        check_should_not_none(account_type, 'accountType')
        check_should_not_none(activation, 'activation')

        check_in_list(account_type,
                      [SubuserTradePrivilegeType.MARGIN, SubuserTradePrivilegeType.SUPER_MARGIN], "accountType")
        check_in_list(activation, [SubUserTradeStatus.ACTIVATED, SubUserTradeStatus.DEACTIVATED], "activation")

        params = {
            'subUids': sub_uids,
            'accountType': account_type,
            'activation': activation
        }
        from huobi.service.subuser.post_tradable_market import PostTradableMarketService
        return PostTradableMarketService(params).request(**self.__kwargs)

    def post_set_subuser_transferability(self, sub_uids: 'str', transferrable: 'bool',
                                         account_type: 'AccountType' = AccountType.SPOT):
        check_should_not_none(sub_uids, 'subUids')
        check_should_not_none(transferrable, 'transferrable')
        check_in_list(account_type, [AccountType.SPOT], 'accountType')

        params = {
            "subUids": sub_uids,
            "accountType": account_type,
            "transferrable": transferrable
        }
        from huobi.service.subuser.post_set_transferability import PostSetSubuserTransferability
        return PostSetSubuserTransferability(params).request(**self.__kwargs)

    def post_subuser_apikey_generate(self, otp_token: 'str', sub_uid: 'int', note: 'str', permission: 'bool',
                                     ip_addresses: 'str' = None):
        check_should_not_none(otp_token, 'otpToken')
        check_should_not_none(sub_uid, 'subUid')
        check_should_not_none(note, 'note')
        check_should_not_none(permission, 'permission')
        # check_in_list(permission, [AccountType.SPOT], 'accountType')

        params = {
            "otpToken": otp_token,
            "subUid": sub_uid,
            "note": note,
            "permission": permission,
            "ipAddresses": ip_addresses
        }
        from huobi.service.subuser.post_subuser_apikey_generation import PostSubuserApikeyGenerationService
        return PostSubuserApikeyGenerationService(params).request(**self.__kwargs)

    def get_user_apikey_info(self, uid: 'str', access_key: 'str' = None):
        check_should_not_none(uid, 'uid')

        params = {
            "uid": uid,
            "accessKey": access_key
        }
        from huobi.service.subuser.get_user_apikey_info import GetUserApikeyInfoService
        return GetUserApikeyInfoService(params).request(**self.__kwargs)

    def post_subuser_apikey_modification(self, sub_uid: 'str', access_key: 'str', note: 'str' = None,
                                         permission: 'str' = None, ip_addresses: 'str' = None):
        check_should_not_none(sub_uid, 'subUid')
        check_should_not_none(access_key, 'accessKey')

        params = {
            "subUid": sub_uid,
            "accessKey": access_key,
            "note": note,
            "permission": permission,
            "ipAddresses": ip_addresses
        }
        from huobi.service.subuser.post_subuser_apikey_modification import PostSubuserApikeyModificationService
        return PostSubuserApikeyModificationService(params).request(**self.__kwargs)

    def post_subuser_apikey_deletion(self, sub_uid: 'str', access_key: 'str'):
        check_should_not_none(sub_uid, 'subUid')
        check_should_not_none(access_key, 'accessKey')

        params = {
            "subUid": sub_uid,
            "accessKey": access_key
        }
        from huobi.service.subuser.post_subuser_apikey_deletion import PostSubuserApikeyDeletionService
        return PostSubuserApikeyDeletionService(params).request(**self.__kwargs)

    def get_uid(self):
        params = {
        }
        from huobi.service.subuser.get_uid import GetUidService
        return GetUidService(params).request(**self.__kwargs)



from huobi.model.generic.symbol import Symbol
from huobi.model.generic.exchange_info import ExchangeInfo
from huobi.model.generic.chain import Chain
from huobi.model.generic.reference_currency import ReferenceCurrency
from huobi.model.generic.market_status import MarketStatus



class ExchangeInfo:
    """
    The Huobi supported the symbols and currencies.

    :member
        symbol_list: The symbol list. The content is Symbol class.
        currencies: The currency list. The content is string value.
    """

    def __init__(self):
        self.symbol_list = list()
        self.currencies = list()




class AccountBalanceReq:
    """
    The account change information received by subscription of account.

    :member
        ts: The UNIX formatted timestamp generated by server in UTC.
        cid: client request ID
        topic: request Channel or Topic
        data: The list of account and balance

    """

    def __init__(self):
        self.ts = 0
        self.cid = ""
        self.topic = ""
        self.data = list()


    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.ts, format_data + "Timestamp")
        PrintBasic.print_basic(self.cid, format_data + "Client Order ID")
        PrintBasic.print_basic(self.topic, format_data + "Topic")
        print()
        if len(self.data):
            for account_balance in self.data:
                account_balance.print_object()
                print()



class MockWebsocketConnection:
    def __init__(self, request):
        self.request = request
        self.send_data_buffer = list()

    def send(self, data):
        self.send_data_buffer.append(data)

    def pop_output_message(self):
        return self.send_data_buffer.pop(0)





from huobi.model.trade.order_list_item import OrderListItem


class OrderDetailReq:
    """
    The order update received by subscription of order update.

    :member
        symbol: The symbol you subscribed.
        timestamp: The UNIX formatted timestamp generated by server in UTC.
        topic : request topic
        client_req_id : client request id
        data: The order detail.

    """

    def __init__(self):
        self.ts = 0
        self.topic = ""
        self.cid = ""
        self.data = OrderListItem()


    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.ts, format_data + "Timestamp")
        PrintBasic.print_basic(self.cid, format_data + "Client Req ID")
        PrintBasic.print_basic(self.topic, format_data + "Topic")
        self.data.print_object("\t")


from huobi.client.account import AccountClient
from huobi.client.trade import TradeClient
from huobi.constant import *
from huobi.utils import *


def callback(order_req_obj: 'OrderListRequest'):
    print("---- order list:  ----")
    order_req_obj.print_object()
    print()

# for test, get spot account
account_client = AccountClient(api_key=g_api_key, secret_key=g_secret_key)
account_spot = account_client.get_account_by_type_and_symbol(account_type=AccountType.SPOT, symbol=None)

# request the order list info
if account_spot and account_spot.id:
    trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key)
    PrintBasic.print_basic(account_spot.id, "Account ID")
    trade_client.req_order_list(symbol="eosusdt", account_id=account_spot.id, callback=callback, order_states = OrderState.CANCELED, client_req_id = "xxx-01-1")





from huobi.utils.print_mix_object import PrintMix, PrintBasic, PrintList
from huobi.utils.channels import *
from huobi.utils.channels_request import *
from huobi.utils.json_parser import default_parse, default_parse_list_dict, default_parse_fill_directly
from huobi.utils.api_signature import create_signature, utc_now
from huobi.utils.api_signature_v2 import create_signature_v2, utc_now
from huobi.utils.url_params_builder import UrlParamsBuilder
from huobi.utils.time_service import get_current_timestamp, convert_cst_in_millisecond_to_utc, convert_cst_in_second_to_utc
from huobi.utils.input_checker import check_symbol, check_symbol_list, check_currency, check_range, check_should_none, \
    check_should_not_none, check_list, greater_or_equal, format_date
from huobi.utils.log_info import LogLevel, LogInfo





class TradeDetailReq:
    """
    The trade received by subscription of trade.

    :member
        rep: The Channel you subscribed.
        trade_list: The trade list. The content is Trade class.
    """

    def __init__(self):
        self.rep = ""
        self.data = list()

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.rep, format_data + "Channel")
        print()
        if len(self.data):
            for trade_detail in self.data:
                trade_detail.print_object()
                print()


from huobi.client.subuser import SubuserClient
from huobi.constant import *
from huobi.utils import *

subuser_client = SubuserClient(api_key=g_api_key, secret_key=g_secret_key)

subUids = '159284259'
accountType = SubuserTradePrivilegeType.MARGIN
activation = SubUserTradeStatus.DEACTIVATED

subUserList = subuser_client.post_set_tradable_market(subUids, accountType, activation)
LogInfo.output_list(subUserList)




from huobi.client.account import AccountClient
from huobi.constant import *
from huobi.utils import *
import time

account_client = AccountClient(api_key=g_api_key,
                              secret_key=g_secret_key)
transfer_order_id = account_client.transfer_between_futures_and_pro(currency="trx", amount=200, transfer_type=TransferFuturesPro.TO_FUTURES)
LogInfo.output("transfer from pro to future Order Id : {id}".format(id=transfer_order_id))

# need wait a minute
time.sleep(2)

transfer_order_id = account_client.transfer_between_futures_and_pro(currency="trx", amount=200, transfer_type=TransferFuturesPro.TO_PRO)
LogInfo.output("transfer from future to pro Order Id : {id}".format(id=transfer_order_id))







from huobi.client.wallet import WalletClient
from huobi.constant import *
from huobi.utils import *


wallet_client = WalletClient(api_key=g_api_key, secret_key=g_secret_key)
list_obj = wallet_client.get_account_withdraw_address(currency="usdt")
LogInfo.output_list(list_obj)








from huobi.client.trade import TradeClient
from huobi.constant import *
from huobi.utils import *


trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key)
list_obj = trade_client.get_match_result(symbol="trxusdt", size=5, direct=QueryDirection.NEXT)
LogInfo.output_list(list_obj)

list_obj = trade_client.get_match_result(symbol="eosusdt", size=5)
LogInfo.output_list(list_obj)


from huobi.model.trade import TradeClearing


class TradeClearingEvent:
    """
    subscribe trading clearing information

    :member
        action: current is "sub" for subscribe
        ch: subscribe topic.
        data: data detail in TradeClearing.
    """

    def __init__(self):
        self.action = ""
        self.ch = ""
        self.seq = 0
        self.data = TradeClearing()

    @staticmethod
    def json_parse(data_json):
        event_obj = TradeClearingEvent()
        event_obj.action = data_json.get("action")
        event_obj.ch = data_json.get("ch")
        event_obj.seq = data_json.get("seq", 0)
        event_obj.data = TradeClearing.json_parse(data_json.get("data", {}))
        return event_obj

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.ch, format_data + "Channel")
        self.data.print_object()




from huobi.client.market import MarketClient
from huobi.utils import *


market_client = MarketClient()
symbol = "btcusdt"
depth_size = 6
depth = market_client.get_pricedepth(symbol, DepthStep.STEP0, depth_size)
LogInfo.output("---- Top {size} bids ----".format(size=len(depth.bids)))
i = 0
for entry in depth.bids:
    i = i + 1
    LogInfo.output(str(i) + ": price: " + str(entry.price) + ", amount: " + str(entry.amount))

LogInfo.output("---- Top {size} asks ----".format(size=len(depth.asks)))

i = 0
for entry in depth.asks:
    i = i + 1
    LogInfo.output(str(i) + ": price: " + str(entry.price) + ", amount: " + str(entry.amount))


from huobi.client.algo import AlgoClient
from huobi.constant import *
from huobi.utils import *

symbol_test = "adausdt"
account_id = g_account_id

algo_client = AlgoClient(api_key=g_api_key, secret_key=g_secret_key)
result = algo_client.get_open_orders()
LogInfo.output_list(result)



from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant import *
from huobi.model.wallet import *
from huobi.utils import *


class GetDepositWithdrawService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/query/deposit-withdraw"

        def parse(dict_data):
            op_type = self.params["type"]
            data_list = dict_data.get("data", [])
            if op_type == DepositWithdraw.DEPOSIT:
                return default_parse_list_dict(data_list, Deposit)
            elif op_type == DepositWithdraw.WITHDRAW:
                return default_parse_list_dict(data_list, Withdraw)
            return []

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)









from huobi.client.account import AccountClient
from huobi.client.trade import TradeClient
from huobi.constant import *
from huobi.utils import *

symbol_test = "eosusdt"


account_id = g_account_id

trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key)
order_id = trade_client.create_order(symbol=symbol_test, account_id=account_id, order_type=OrderType.BUY_LIMIT, source=OrderSource.API, amount=4.0, price=1.292)
LogInfo.output("created order id : {id}".format(id=order_id))

canceled_order_id = trade_client.cancel_order(symbol_test, order_id)
if canceled_order_id == order_id:
    LogInfo.output("cancel order {id} done".format(id=canceled_order_id))
else:
    LogInfo.output("cancel order {id} fail".format(id=canceled_order_id))

order_id = trade_client.create_order(symbol=symbol_test, account_id=account_id, order_type=OrderType.BUY_MARKET, source=OrderSource.API, amount=5.0, price=1.292)
LogInfo.output("created order id : {id}".format(id=order_id))

order_id = trade_client.create_order(symbol=symbol_test, account_id=account_id, order_type=OrderType.SELL_MARKET, source=OrderSource.API, amount=1.77, price=None)
LogInfo.output("created order id : {id}".format(id=order_id))




from huobi.client.generic import GenericClient
from huobi.utils import *


generic_client = GenericClient()

list_obj = generic_client.get_reference_currencies()
LogInfo.output_list(list_obj)


list_obj = generic_client.get_reference_currencies(currency="usdt")
LogInfo.output_list(list_obj)




from huobi.client.algo import AlgoClient
from huobi.constant import *
from huobi.utils import *

symbol_test = "adausdt"
account_id = g_account_id

orders_to_cancel = ["test003", "test001"]
algo_client = AlgoClient(api_key=g_api_key, secret_key=g_secret_key)
result = algo_client.cancel_orders(orders_to_cancel)
result.print_object()

# order_id = algo_client.create_order(symbol=symbol_test, account_id=account_id, order_type=OrderType.BUY_MARKET, source=OrderSource.API, amount=5.0, price=1.292)
# LogInfo.output("created order id : {id}".format(id=order_id))
#
# order_id = algo_client.create_order(symbol=symbol_test, account_id=account_id, order_type=OrderType.SELL_MARKET, source=OrderSource.API, amount=1.77, price=None)
# LogInfo.output("created order id : {id}".format(id=order_id))



from huobi.client.generic import GenericClient


"""
GET https://status.huobigroup.com/api/v2/summary.json
"""
generic_client = GenericClient()
system_status = generic_client.get_system_status()
print(system_status)






from huobi.constant import *


class AccountUpdate:
    """
    The account change information received by subscription of account.

    :member
        currency: The currency of the change.
        accountId: The account id.
        balance: Account balance (only exists when account balance changed)
        available:	Available balance (only exists when available balance changed)
        changeType:	Change type see AccountChangeType, valid value: order-place,order-match,order-refund,order-cancel,order-fee-refund,margin-transfer,margin-loan,margin-interest,margin-repay,other,
        accountType: Account type see AccountBalanceUpdateType, valid value: trade, frozen, loan, interest
        changeTime:	Change time, unix time in millisecond
    """

    def __init__(self):
        self.currency = ""
        self.accountId = 0
        self.balance = ""
        self.available = ""
        self.changeType = AccountChangeType.INVALID
        self.accountType = AccountBalanceUpdateType.INVALID
        self.changeTime = 0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.accountId, format_data + "Account ID")
        PrintBasic.print_basic(self.balance, format_data + "Balance")
        PrintBasic.print_basic(self.available, format_data + "Available")
        PrintBasic.print_basic(self.changeType, format_data + "Account Change Type")
        PrintBasic.print_basic(self.accountType, format_data + "Account Balance Change Type")
        PrintBasic.print_basic(self.changeTime, format_data + "Account Timestamp")



from huobi.constant import *
from huobi.model.account.account_point_group import AccountPointGroup
from huobi.utils import default_parse, default_parse_list_dict


class AccountPointResult:
    """
    The account information for spot account, margin account etc.

    :member
        id: The unique account id.
        account_type: The type of this account, possible value: spot, margin, otc, point.
        account_state: The account state, possible value: working, lock.
        list: The balance list of the specified currency. The content is Balance class

    """

    def __init__(self):
        self.accountId = ""
        self.accountStatus = AccountPointState.INVALID
        self.groupIds = list()
        self.acctBalance = ""

    @staticmethod
    def json_parse(data_dict):
        if data_dict and len(data_dict):
            group_ids = data_dict.get("groupIds")
            data_dict.pop("groupIds")
            account_point_obj = default_parse(data_dict, AccountPointResult, AccountPointGroup)
            account_point_obj.subtype = data_dict.get("subtype", data_dict.get("symbol"))
            account_point_obj.list = default_parse_list_dict(group_ids, AccountPointGroup, [])
            return account_point_obj

        return None

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.accountId, format_data + "Account ID")
        PrintBasic.print_basic(self.accountStatus, format_data + "Account Status")
        PrintBasic.print_basic(self.acctBalance, format_data + "Account Balance")

        print()
        if len(self.groupIds):
            for row in self.groupIds:
                row.print_object(format_data + "\t")
                print()




from huobi.client.trade import TradeClient
from huobi.constant import *


def callback(order_req_obj: 'OrderDetailReq'):
    order_req_obj.print_object()

order_id = 25943298391691
trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key)
trade_client.req_order_detail(order_id, callback, client_req_id="abcde")






from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant.system import HttpMethod
from huobi.model.market import *
from huobi.utils import *


class GetHistoryTradeService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/market/history/trade"

        def parse(dict_data):
            trade_list_ret = []  # two level list, list item is list too
            data_list_outer = dict_data.get("data", [])
            if len(data_list_outer):
                for row in data_list_outer:
                    data_list_inner = row.get("data", [])
                    if len(data_list_inner):
                        for trade_info in data_list_inner:
                            trade_obj = default_parse_list_dict(trade_info, Trade, None)  # return a list
                            if trade_obj:
                                trade_list_ret.append(trade_obj)

            return trade_list_ret

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET, channel, self.params, parse)









from huobi.constant import *


class AccountPointGroup:
    """
    The account information for spot account, margin account etc.

    :member
        id: The unique account id.
        account_type: The type of this account, possible value: spot, margin, otc, point.
        account_state: The account state, possible value: working, lock.
        list: The balance list of the specified currency. The content is Balance class

    """

    def __init__(self):
        self.groupId = ""
        self.expiryDate = ""
        self.remainAmt = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.groupId, format_data + "Group Id")
        PrintBasic.print_basic(self.expiryDate, format_data + "Expiration date")
        PrintBasic.print_basic(self.remainAmt, format_data + "Remain Amount")



import time


from huobi.connection.subscribe_client import SubscribeClient
from huobi.model.trade import *
from huobi.utils import *


class SubTradeClearingV2Service:
    def __init__(self, params):
        self.params = params

    def subscribe(self, callback, error_handler, **kwargs):
        symbol_list = self.params["symbol_list"]

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(trade_clearing_channel(symbol))
                time.sleep(0.01)

        def parse(dict_data):
            return TradeClearingEvent.json_parse(dict_data)

        SubscribeClient(**kwargs).execute_subscribe_v2(subscription,
                                            parse,
                                            callback,
                                            error_handler,
                                            is_trade=True)











class ChainDepositAddress:
    """
    The deposit address.

    :member
        currency: The crypto currency to deposit.
        address: Deposit address
        addressTag: Deposit address tag.
        chain: Block chain name.
    """
    def __init__(self):
        self.currency = ""
        self.address = ""
        self.addressTag = ""
        self.chain = ""



    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.address, format_data + "Address")
        PrintBasic.print_basic(self.addressTag, format_data + "addressTag")
        PrintBasic.print_basic(self.chain, format_data + "Chain")



from huobi.client.account import AccountClient
from huobi.constant import *

account_client = AccountClient(api_key=g_api_key,
                               secret_key=g_secret_key)

account_point_result = account_client.get_account_point()
account_point_result.print_object()



from huobi.constant import *
from huobi.model.account.balance import Balance
from huobi.utils import default_parse, default_parse_list_dict


class AccountBalance:
    """
    The account information for spot account, margin account etc.

    :member
        id: The unique account id.
        account_type: The type of this account, possible value: spot, margin, otc, point.
        account_state: The account state, possible value: working, lock.
        list: The balance list of the specified currency. The content is Balance class

    """

    def __init__(self):
        self.id = 0
        self.type = AccountType.INVALID
        self.state = AccountState.INVALID
        self.subtype = ""
        self.list = list()

    @staticmethod
    def json_parse(data_dict):
        if data_dict and len(data_dict):
            balance_list = data_dict.get("list")
            data_dict.pop("list")
            account_balance_obj = default_parse(data_dict, AccountBalance, Balance)
            account_balance_obj.subtype = data_dict.get("subtype", data_dict.get("symbol"))
            account_balance_obj.list = default_parse_list_dict(balance_list, Balance, [])
            return account_balance_obj

        return None

    @staticmethod
    def json_parse_list(data_list):
        account_balance_list = []
        if data_list and len(data_list):
            for item in data_list:
                item_obj = AccountBalance.json_parse(item)
                if item_obj:
                    account_balance_list.append(item_obj)
        return account_balance_list

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "Account ID")
        PrintBasic.print_basic(self.type, format_data + "Account Type")
        PrintBasic.print_basic(self.state, format_data + "Account State")
        PrintBasic.print_basic(self.subtype, format_data + "Subtype")

        print()
        if len(self.list):
            for row in self.list:
                row.print_object(format_data+"\t")
                print()




from huobi.constant import *


class CompleteSubAccount:
    """
    Sub-account completed info

    :member
        id: The sub-id.
        account_type: The sub account type.
        balances: The balance list, the content is Balance class.
    """

    def __init__(self):
        self.id = 0
        self.account_type = AccountType.INVALID
        self.balances = list()

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "ID")
        PrintBasic.print_basic(self.account_type, format_data + "Account Type")
        if len(self.balances):
            for row in self.balances:
                row.print_object()
                print()


from huobi.client.account import AccountClient
from huobi.constant import *

account_client = AccountClient(api_key=g_api_key, secret_key=g_secret_key)
# transfer_result = account_client.post_account_transfer(111859319, 'spot', 10354000, 147509118, 'spot', 13956126, 'trx', 3600)
# transfer_result.print_object()

# transfer_result = account_client.post_account_transfer(111859319, 'spot', 10354000, 147991959, 'spot', 14026125, 'usdt', 3600)
# transfer_result.print_object()

transfer_result = account_client.post_account_transfer(111859319, 'spot', 10354000, 141403659, 'spot', 13082796, 'usdt', 32.21)
transfer_result.print_object()



from huobi.constant import *


class Withdraw:
    """
    The latest status for withdraws.

    :member
        id: The transfer id.
        currency: The crypto currency to deposit.
        tx_hash: The on-chain transaction hash.
        amount: The number of crypto asset transferred in its minimum unit.
        address: The deposit source address.
        address_tag: The user defined address tag.
        fee: The amount of fee taken by Huobi in this crypto's minimum unit.
        created_at: The UNIX formatted timestamp in UTC for the transfer creation.
        updated_at: The UNIX formatted timestamp in UTC for the transfer's latest update.
        state: The withdraw state of this transfer.
    """
    def __init__(self):
        self.id = 0
        self.type = DepositWithdraw.WITHDRAW
        self.currency = ""
        self.chain = ""
        self.tx_hash = ""
        self.amount = 0.0
        self.address = ""
        self.address_tag = ""
        self.fee = 0.0
        self.created_at = 0
        self.updated_at = 0
        self.state = WithdrawState.INVALID


    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "ID")
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.type, format_data + "Operator Type")
        PrintBasic.print_basic(self.chain, format_data + "Chain")
        PrintBasic.print_basic(self.tx_hash, format_data + "Trade Hash")
        PrintBasic.print_basic(self.amount, format_data + "Amount")
        PrintBasic.print_basic(self.address, format_data + "Address")
        PrintBasic.print_basic(self.address_tag, format_data + "Address Tag")
        PrintBasic.print_basic(self.fee, format_data + "Fee")
        PrintBasic.print_basic(self.state, format_data + "Withdraw State")
        PrintBasic.print_basic(self.created_at, format_data + "Create Time")
        PrintBasic.print_basic(self.updated_at, format_data + "Update Time")


from huobi.client.market import MarketClient
from huobi.constant import *
from huobi.utils import *

market_client = MarketClient(init_log=True)
list_obj = market_client.get_market_tickers()
LogInfo.output_list(list_obj)




















class SubuserCreation:
    """
    The trade information with price and amount etc.

    :member
        subUid: sub user ID.
        userState: sub user account state, states see SubUidState.
    """

    def __init__(self):
        self.user_name = ""
        self.note = ""
        self.uid = 0
        self.err_code = 0
        self.err_message = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.user_name, format_data + "userName")
        PrintBasic.print_basic(self.note, format_data + "note")
        PrintBasic.print_basic(self.uid, format_data + "uid")
        PrintBasic.print_basic(self.err_code, format_data + "errCode")
        PrintBasic.print_basic(self.err_message, format_data + "errMessage")






from huobi.constant import *


class DepositHistoryItem:
    """
     The deposit history

     :member
         id: The transfer id.
         currency: The crypto currency to deposit.
         txHash: The on-chain transaction hash.
         amount: The number of crypto asset transferred in its minimum unit.
         address: The deposit source address.
         addressTag: The user defined address tag.
         deposit_state: The deposit state of this transfer.
         created_timestamp: The UNIX formatted timestamp in UTC for the transfer creation.
         updated_timestamp: The UNIX formatted timestamp in UTC for the transfer's latest update.
     """
    def __init__(self):
        self.id = 0
        self.currency = ""
        self.txHash = ""
        self.chain = ""
        self.amount = 0.0
        self.address = ""
        self.addressTag = ""
        self.deposit_state = WithdrawState.INVALID
        self.created_timestamp = 0
        self.updated_timestamp = 0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "ID")
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.chain, format_data + "Chain")
        PrintBasic.print_basic(self.txHash, format_data + "Trade Hash")
        PrintBasic.print_basic(self.amount, format_data + "Amount")
        PrintBasic.print_basic(self.address, format_data + "Address")
        PrintBasic.print_basic(self.addressTag, format_data + "Address Tag")
        PrintBasic.print_basic(self.deposit_state, format_data + "Deposit State")
        PrintBasic.print_basic(self.created_timestamp, format_data + "Create Time")
        PrintBasic.print_basic(self.updated_timestamp, format_data + "Update Time")



class CancelOrderResult:
    """
    The result of batch cancel operation.

    :member
        accepted: The clientOrderIds accepted.
        rejected: The clientOrderIds rejected .

    """

    def __init__(self):
        self.accepted = []
        self.rejected = []

    def print_object(self, format_data=""):
        print("Success Order Counts", len(self.accepted), " accepted Order Ids : ", self.accepted)
        print("Fail Order Counts", len(self.rejected), " Rejected Order Ids : ", self.rejected)


from huobi.client.margin import MarginClient
from huobi.constant import *
from huobi.utils import *


amount = 100
currency = "usdt"


margin_client = MarginClient(api_key=g_api_key, secret_key=g_secret_key)
# order_id = margin_client.post_cross_margin_create_loan_orders(currency=currency, amount=amount)
# LogInfo.output("step 1: create loan order {id}".format(id=order_id))

# to check not clearing loan orders
list_obj = margin_client.get_cross_margin_loan_orders(currency=currency, state=LoanOrderState.ACCRUAL)
LogInfo.output("step 2: loaning order information ")
LogInfo.output_list(list_obj)

if list_obj and len(list_obj):
    for loan_order in list_obj:
        # pay attention to Scientific Notation
        repay_amount = float(loan_order.loan_balance) + float(loan_order.interest_balance)
        LogInfo.output("repay loan order {id}, repay amount : {amount}".format(id=loan_order.id, amount=repay_amount))
        result = margin_client.post_cross_margin_loan_order_repay(order_id=loan_order.id, amount=repay_amount)
        LogInfo.output("step 3: repay loan order {id}, status : {status}, repay amount : {amount}".format(id=loan_order.id, status=result, amount=repay_amount))

list_obj = margin_client.get_cross_margin_loan_orders(currency=currency, state=LoanOrderState.ACCRUAL)
LogInfo.output("step 4: loaning order history ")
LogInfo.output_list(list_obj)



from huobi.client.trade import TradeClient
from huobi.constant import *
from huobi.utils import *
import time

symbol_test = "eosusdt"
client_order_id_header = str(int(time.time()))
client_order_id_test = "client_" + client_order_id_header +"_order"  # unique id in 24hours

account_id = g_account_id
trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key)
order_id = trade_client.create_order(symbol=symbol_test,
                                     account_id=account_id,
                                     order_type=OrderType.BUY_LIMIT,
                                     source=OrderSource.API,
                                     amount=20,
                                     price=0.26,
                                     client_order_id=client_order_id_test,
                                     stop_price=0.11,
                                     operator="gte")
LogInfo.output("======= create new order id : {order_id} with client id {client_id} =======".format(order_id=(order_id),
                                                                                 client_id=client_order_id_test))

orderObj = trade_client.get_order(order_id=order_id)
LogInfo.output("======= get order by order id : {order_id} =======".format(order_id=order_id))
orderObj.print_object()

orderObj = trade_client.get_order_by_client_order_id(client_order_id=client_order_id_test)
LogInfo.output("======= get order by client order id : {client_id} =======".format(client_id=client_order_id_test))
orderObj.print_object()

trade_client.cancel_client_order(client_order_id=client_order_id_test)
LogInfo.output("======= cancel order by client order id : {client_id} =======".format(client_id=client_order_id_test))

orderObj = trade_client.get_order_by_client_order_id(client_order_id=client_order_id_test)
LogInfo.output("======= get order by client order id : {client_id} after cancel =======".format(client_id=client_order_id_test))
orderObj.print_object()



from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant.system import HttpMethod


class GetSystemStatusService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/api/v2/summary.json"
        kwargs["url"] = "https://status.huobigroup.com"

        def parse(dict_data):
            return dict_data

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET, channel, self.params, parse)









import time

from huobi.model.market import *
from huobi.utils import *
from huobi.connection.subscribe_client import SubscribeClient


class SubPriceDepthService:
    def __init__(self, params):
        self.params = params

    def subscribe(self, callback, error_handler, **kwargs):
        symbol_list = self.params["symbol_list"]
        step = self.params["step"]

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(price_depth_channel(symbol, step))
                time.sleep(0.01)

        def parse(dict_data):
            price_depth_event_obj = PriceDepthEvent()
            price_depth_event_obj.ch = dict_data.get("ch", "")
            tick = dict_data.get("tick", "")
            price_depth_event_obj.tick = PriceDepth.json_parse(tick)
            return price_depth_event_obj

        SubscribeClient(**kwargs).execute_subscribe_v1(subscription,
                                            parse,
                                            callback,
                                            error_handler)






from huobi.constant import *
from huobi.model.etf.unitprice import UnitPrice
from huobi.utils import default_parse_list_dict


class EtfSwapList:
    """
    The past creation and redemption.

    :member
        id: the operation Id.
        gmt_created: The UNIX formatted timestamp in UTC of the operation.
        currency: The ETF name.
        amount: Creation or redemption amount.
        type: The swap type. Creation or redemption.
        status: The operation result
        rate: The fee rate.
        fee: The actual fee amount

        point_card_amount: Discount from point card.
        used_currency_list: For creation this is the list and amount of underlying assets used for ETF creation.
            For redemption this is the amount of ETF used for redemption. The content is UnitPrice class.
        obtain_currency_list: For creation this is the amount for ETF created.
            For redemption this is the list and amount of underlying assets obtained. The content is UnitPrice class
    """

    def __init__(self):
        self.id = 0
        self.gmt_created = 0
        self.currency = ""
        self.amount = 0.0
        self.type = EtfSwapType.INVALID
        self.status = 0
        self.rate = 0.0
        self.fee = 0.0
        self.point_card_amount = 0.0
        self.used_currency_list = list()
        self.obtain_currency_list = list()

    @staticmethod
    def json_parse(dict_data):
        if dict_data and len(dict_data):
            detail = dict_data.get("detail", {})
            dict_data.pop("detail")
            etf_swap_obj = default_parse_list_dict(dict_data, EtfSwapList)
            if detail and len(detail):
                etf_swap_obj.rate = detail.get("rate", 0)
                etf_swap_obj.fee = detail.get("fee", 0)
                etf_swap_obj.point_card_amount = detail.get("point_card_amount", 0)
                etf_swap_obj.used_currency_list = default_parse_list_dict(detail.get("used_currency_list"), UnitPrice, [])
                etf_swap_obj.obtain_currency_list = default_parse_list_dict(detail.get("obtain_currency_list"), UnitPrice, [])
            return etf_swap_obj

        return None

    @staticmethod
    def json_parse_list(dict_data):
        ret_list = list()
        for item in dict_data:
            item_obj = EtfSwapList.json_parse(item)
            if item_obj is not None:
                ret_list.append(item_obj)

        return ret_list


    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "Operater Id")
        PrintBasic.print_basic(self.gmt_created, format_data + "GMT Create Time")
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.type, format_data + "Type")
        PrintBasic.print_basic(self.amount, format_data + "Amount")
        PrintBasic.print_basic(self.rate, format_data + "Rate")
        PrintBasic.print_basic(self.fee, format_data + "Fee")
        PrintBasic.print_basic(self.status, format_data + "Status")
        PrintBasic.print_basic(self.point_card_amount, format_data + "Point Card Amount")

        if len(self.used_currency_list):
            PrintBasic.print_basic("used_currency_list as below:")
            for row in self.used_currency_list:
                row.print_object(format_data + "\t")

        if len(self.obtain_currency_list):
            PrintBasic.print_basic("obtain_currency_list as below:")
            for row in self.obtain_currency_list:
                row.print_object(format_data + "\t")




class Symbol:
    """
    The Huobi supported symbols.

    :member
        base_currency: The base currency in a trading symbol.
        quote_currency: The quote currency in a trading symbol.
        price_precision: The quote currency precision when quote price (decimal places).
        amount_precision: The base currency precision when quote amount (decimal places).
        symbol_partition: The trading section, possible values: [main，innovation，bifurcation].
        symbol: The symbol, like "btcusdt".
        state : trade status, maybe one in [online，offline,suspend]
        value_precision : value precision
        min_order_amt : minimum volume limit only used in limit-order and sell-market order
        max_order_amt : Maximum volume
        min_order_value : Minimum order amount
        leverage_ratio : Leverage ratio for symbol
        limit_order_min_order_amt: Minimum order amount of limit order in base currency (NEW)
        limit_order_max_order_amt: Max order amount of limit order in base currency (NEW)
        sell_market_min_order_amt: Minimum order amount of sell-market order in base currency (NEW)
        sell_market_max_order_amt: Max order amount of sell-market order in base currency (NEW)
        buy_market_max_order_amt: Max order value of buy-market order in quote currency (NEW)
        max_order_value: Max order value of limit order and buy-market order in usdt (NEW)

    """

    def __init__(self):
        self.base_currency = ""
        self.quote_currency = ""
        self.price_precision = 0
        self.amount_precision = 0
        self.symbol_partition = ""
        self.symbol = ""
        self.state = ""
        self.value_precision = 0
        self.min_order_amt = ""
        self.max_order_amt = ""
        self.min_order_value = ""
        self.leverage_ratio = 0
        self.limit_order_min_order_amt = 0
        self.limit_order_max_order_amt = 0
        self.sell_market_min_order_amt = 0
        self.sell_market_max_order_amt = 0
        self.buy_market_max_order_value = 0
        self.max_order_value = 0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.base_currency, format_data + "Base Currency")
        PrintBasic.print_basic(self.quote_currency, format_data + "Quote Currency")
        PrintBasic.print_basic(self.price_precision, format_data + "Price Precision")
        PrintBasic.print_basic(self.amount_precision, format_data + "Amount Precision")
        PrintBasic.print_basic(self.symbol_partition, format_data + "Symbol Partition")
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        PrintBasic.print_basic(self.state, format_data + "State")
        PrintBasic.print_basic(self.value_precision, format_data + "Value Precision")
        PrintBasic.print_basic(self.min_order_amt, format_data + "Min Order Amount")
        PrintBasic.print_basic(self.max_order_amt, format_data + "Max Order Amount")
        PrintBasic.print_basic(self.min_order_value, format_data + "Min Order Value")
        PrintBasic.print_basic(self.leverage_ratio, format_data + "Leverage Ratio")
        PrintBasic.print_basic(self.limit_order_min_order_amt, format_data + "Minimum order amount (Limit Order)")
        PrintBasic.print_basic(self.limit_order_max_order_amt, format_data + "Max order amount (Limit Order)")
        PrintBasic.print_basic(self.sell_market_min_order_amt, format_data + "Min order amount (Sell Market Order)")
        PrintBasic.print_basic(self.sell_market_max_order_amt, format_data + "Max order amount (Sell Market Order)")
        PrintBasic.print_basic(self.buy_market_max_order_value, format_data + "Max order value (Buy Market Order)")
        PrintBasic.print_basic(self.max_order_value, format_data + "Max order value (In USDT)")



from huobi.client.trade import TradeClient
from huobi.constant import *
from huobi.utils import *

client_order_id_test = "xxxxxx"  # unique id in 24hours

trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key)
orderObj = trade_client.get_order_by_client_order_id(client_order_id=client_order_id_test)
LogInfo.output("======= get order by client order id : {client_id} =======".format(client_id=client_order_id_test))
orderObj.print_object()





class OrderHistoryItem:
    """
    The result of batch cancel operation.

    :member
        orderOrigTime
        lastActTime
        symbol
        source
        orderSide
        orderType
        timeInForce
        clientOrderId
        accountId
        orderPrice
        orderSize
        stopPrice
        orderStatus

    """

    def __init__(self):
        self.orderOrigTime = ""
        self.lastActTime = ""
        self.symbol = ""
        self.source = ""
        self.orderSide = ""
        self.orderType = ""
        self.timeInForce = ""
        self.clientOrderId = ""
        self.accountId = ""
        self.orderPrice = ""
        self.orderSize = ""
        self.stopPrice = ""
        self.orderStatus = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.orderOrigTime, format_data + "")
        PrintBasic.print_basic(self.lastActTime, format_data + "")
        PrintBasic.print_basic(self.symbol, format_data + "")
        PrintBasic.print_basic(self.source, format_data + "")
        PrintBasic.print_basic(self.orderSide, format_data + "")
        PrintBasic.print_basic(self.orderType, format_data + "")
        PrintBasic.print_basic(self.timeInForce, format_data + "")
        PrintBasic.print_basic(self.clientOrderId, format_data + "")
        PrintBasic.print_basic(self.accountId, format_data + "")
        PrintBasic.print_basic(self.orderPrice, format_data + "")
        PrintBasic.print_basic(self.orderSize, format_data + "")
        PrintBasic.print_basic(self.stopPrice, format_data + "")
        PrintBasic.print_basic(self.orderStatus, format_data + "")



from huobi.model.account.account import Account
from huobi.model.account.account_balance import AccountBalance
from huobi.model.account.account_balance_req import AccountBalanceReq
from huobi.model.account.account_update import AccountUpdate
from huobi.model.account.account_update_event import AccountUpdateEvent
from huobi.model.account.balance import Balance
from huobi.model.account.complete_subaccount import CompleteSubAccount
from huobi.model.account.margin_balance_detail import MarginBalanceDetail
from huobi.model.account.account_history import AccountHistory
from huobi.model.account.sub_uid_management import SubUidManagement
from huobi.model.account.account_ledger import AccountLedger
from huobi.model.account.account_transfer_result import AccountTransferResult
from huobi.model.account.account_point_result import AccountPointResult
from huobi.model.account.account_point_transfer_result import AccountPointTransferResult



from huobi.model.market import Mbp


class MbpFullEvent:
    """
    full price depth.

    :member
        ch: Topic of subscribed.
        timestamp: The UNIX formatted timestamp generated by server in UTC.
        data: The price depth.

    """

    def __init__(self):
        self.ch = ""
        self.ts = 0
        self.data = Mbp()

    @staticmethod
    def json_parse(json_data):
        mbp_event = MbpFullEvent()
        mbp_event.ts = json_data.get("ts")
        mbp_event.ch = json_data.get("ch")
        mbp = Mbp.json_parse(json_data.get("tick", {}))
        mbp_event.data = mbp
        return mbp_event

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.ch, format_data + "Topic")
        PrintBasic.print_basic(self.ts, format_data + "Timestamp")
        self.data.print_object(format_data + "\t")



class OrderListReq:
    """
    The order update received by request of order list.

    :member
        symbol: The symbol you subscribed.
        timestamp: The UNIX formatted timestamp generated by server in UTC.
        topic: request topic
        data : OrderListItem

    """

    def __init__(self):
        self.ts = 0
        self.topic = ""
        self.data = list()


    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.ts, format_data + "Timestamp")
        PrintBasic.print_basic(self.topic, format_data + "Channel")
        print("Order List as below : count " + str(len(self.data)))
        if len(self.data):
            for orderlistitem_obj in self.data:
                orderlistitem_obj.print_object("\t ")
                print()





from huobi.constant import *

class OrderListItem:
    """
    The order update received by request of order list.

    :member
        symbol: The symbol you subscribed.
        timestamp: The UNIX formatted timestamp generated by server in UTC.
        topic: request topic
        client_req_id: client request ID
        order_list : order list

    """

    def __init__(self):
        self.id = 0
        self.symbol = ""
        self.account_id = 0
        self.amount = 0.0
        self.price = 0.0
        self.created_at = 0
        self.type = OrderType.INVALID
        self.finished_at = 0
        self.source = OrderSource.INVALID
        self.state = OrderState.INVALID
        self.canceled_at = 0
        self.filled_amount = 0.0
        self.filled_cash_amount = 0.0
        self.filled_fees = 0.0
        self.stop_price = 0.0
        self.operator = ""


    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "ID")
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        PrintBasic.print_basic(self.account_id, format_data + "Account Id")
        PrintBasic.print_basic(self.amount, format_data + "Amount")
        PrintBasic.print_basic(self.price, format_data + "Price")
        PrintBasic.print_basic(self.created_at, format_data + "Create Time")
        PrintBasic.print_basic(self.type, format_data + "Order Type")
        PrintBasic.print_basic(self.finished_at, format_data + "Finish Time")
        PrintBasic.print_basic(self.source, format_data + "Order Source")
        PrintBasic.print_basic(self.state, format_data + "Order State")
        PrintBasic.print_basic(self.canceled_at, format_data + "Cancel Time")
        PrintBasic.print_basic(self.filled_amount, format_data + "Filled Amount")
        PrintBasic.print_basic(self.filled_cash_amount, format_data + "Filled Cash Amount")
        PrintBasic.print_basic(self.filled_fees, format_data + "Filled Fees")
        PrintBasic.print_basic(self.stop_price, format_data + "Stop Price")
        PrintBasic.print_basic(self.operator, format_data + "Operator")






class RestApiRequest(object):

    def __init__(self):
        self.method = ""
        self.url = ""
        self.host = ""
        self.post_body = ""
        self.header = dict()
        self.json_parser = None




from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant import *


class PostCreateOrderService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/algo-orders"

        # {'code': 200, 'data': {'clientOrderId': 'test001'}}
        def parse(dict_data):
            data = dict_data.get('data')
            return data.get('clientOrderId')

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)



from huobi.constant import *


class Account:
    """
    The account information for spot account, margin account etc.

    :member
        id: The unique account id.
        account_type: The type of this account, possible value: spot, margin, otc, point.
        account_state: The account state, possible value: working, lock.
        balances: The balance list of the specified currency. The content is Balance class

    """

    def __init__(self):
        self.id = 0
        self.type = AccountType.INVALID
        self.state = AccountState.INVALID
        self.subtype = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "ID")
        PrintBasic.print_basic(self.type, format_data + "Account Type")
        PrintBasic.print_basic(self.state, format_data + "Account State")
        PrintBasic.print_basic(self.subtype, format_data + "Subtype")




from huobi.client.market import MarketClient
from huobi.constant import DepthStep


def callback(price_depth_req: 'PriceDepthReq'):
    price_depth_req.print_object()


def error(e: 'HuobiApiException'):
    print(e.error_code + e.error_message)


sub_client = MarketClient()
sub_client.req_pricedepth("btcusdt", DepthStep.STEP0, callback, error)



from huobi.client.margin import MarginClient
from huobi.constant import *
from huobi.utils import *

margin_client = MarginClient(api_key=g_api_key, secret_key=g_secret_key)
transfer_id = margin_client.post_cross_margin_transfer_in(currency="eos", amount=5)
LogInfo.output(transfer_id)

transfer_id = margin_client.post_cross_margin_transfer_out(currency="eos", amount=5)
LogInfo.output(transfer_id)





class UserApikeyInfo:
    """
    The trade information with price and amount etc.

    :member
        accessKey: .
        createTime:
        ipAddresses: .
        note:
        permission:
        status:
        updateTime:
        validDays:

    """

    def __init__(self):
        self.accessKey = ""
        self.createTime = 0
        self.ipAddresses = ""
        self.note = ""
        self.permission = ""
        self.status = ""
        self.updateTime = 0
        self.validDays = -1

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic

        PrintBasic.print_basic(self.accessKey, format_data + "accessKey")
        PrintBasic.print_basic(self.createTime, format_data + "createTime")
        PrintBasic.print_basic(self.ipAddresses, format_data + "ipAddresses")
        PrintBasic.print_basic(self.note, format_data + "note")
        PrintBasic.print_basic(self.permission, format_data + "permission")
        PrintBasic.print_basic(self.status, format_data + "status")
        PrintBasic.print_basic(self.updateTime, format_data + "updateTime")
        PrintBasic.print_basic(self.validDays, format_data + "validDays")



from huobi.client.account import AccountClient
from huobi.constant import *
from huobi.utils import LogInfo



api_key ='88fbec25-de8461a1-ntmuw4rrsr-bb051'
api_secret ='bde897ae-beb99d2d-cfa29656-1a0a6'



account_client = AccountClient(api_key=api_key,
                               secret_key=api_secret)
LogInfo.output("====== (SDK encapsulated api) not recommend for low performance and frequence limitation ======")
account_balance_list = account_client.get_account_balance()
if account_balance_list and len(account_balance_list):
    for account_obj in account_balance_list:
        account_obj.print_object()
        print()



class LoanInfo:
    """
    The margin rate define.

    :member
        currency: The currency name.
        interest_rate: all interest rate
        min_loan_amt: min loan amount.
        max_loan_amt: max loan amount.
        loanable_amt: loanable amount.
        actual_rate: rate after deduction.
    """

    def __init__(self):
        self.currency = ""
        self.interest_rate = ""
        self.min_loan_amt = ""
        self.max_loan_amt = ""
        self.loanable_amt = ""
        self.actual_rate = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.interest_rate, format_data + "Interest Rate")
        PrintBasic.print_basic(self.min_loan_amt, format_data + "Min Loan Amount")
        PrintBasic.print_basic(self.max_loan_amt, format_data + "Max Loan Amount")
        PrintBasic.print_basic(self.loanable_amt, format_data + "Loanable Amount")
        PrintBasic.print_basic(self.actual_rate, format_data + "Actual Rate")



from huobi.client.trade import TradeClient
from huobi.constant import *
from huobi.utils import *



order_id_test = 87939085540

trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key)
list_obj = trade_client.get_match_results_by_order_id(order_id=order_id_test)
LogInfo.output_list(list_obj)


from huobi.client.margin import MarginClient
from huobi.constant import *
from huobi.utils import *
import time

loan_amount = 100
interest_amount = 0.004083

margin_client = MarginClient(api_key=g_api_key, secret_key=g_secret_key)
loan_id = margin_client.post_create_margin_order(symbol="eosusdt", currency="usdt", amount=loan_amount)
LogInfo.output("step 1: loan id : {id}".format(id=loan_id))

time.sleep(2)

repay_id = margin_client.post_repay_margin_order(loan_id=loan_id, amount=loan_amount + interest_amount)
LogInfo.output("step 2: repay id : {id}".format(id=repay_id))

list_obj = margin_client.get_margin_loan_orders(symbol="eosusdt", states=LoanOrderState.ACCRUAL)
LogInfo.output("step 3: loaning order information")
LogInfo.output_list(list_obj)


list_obj = margin_client.get_margin_loan_orders(symbol="eosusdt")
LogInfo.output("step 4: loan order history")
LogInfo.output_list(list_obj)


from huobi.constant import *

class Chain:
    """
    The Huobi Chain.

    :member
        chain: Chain name
        numOfConfirmations: Number of confirmations required for deposit success (trading & withdrawal allowed once reached)
        numOfFastConfirmations: Number of confirmations required for quick success (trading allowed but withdrawal disallowed once reached)
        minDepositAmt: Minimal deposit amount in each request
        depositStatus: Deposit status	allowed,prohibited
        minWithdrawAmt: Minimal withdraw amount in each request.
        maxWithdrawAmt : Maximum withdraw amount in each request
        withdrawQuotaPerDay : Maximum withdraw amount in a day
        withdrawQuotaPerYear : Maximum withdraw amount in a year
        withdrawQuotaTotal : Maximum withdraw amount in total
        withdrawPrecision : Withdraw amount precision
        withdrawFeeType : Type of withdraw fee (only one type can be applied to each currency)

        transactFeeWithdraw : Withdraw fee in each request (only applicable to withdrawFeeType = fixed)
        minTransactFeeWithdraw : Minimal withdraw fee in each request (only applicable to withdrawFeeType = circulated)
        maxTransactFeeWithdraw : Maximum withdraw fee in each request (only applicable to withdrawFeeType = circulated or ratio)
        transactFeeRateWithdraw : Withdraw fee in each request (only applicable to withdrawFeeType = ratio)
        withdrawStatus : Withdraw status
    """

    def __init__(self):
        self.chain = ""
        self.baseChain = ""
        self.baseChainProtocol = ""
        self.numOfConfirmations = 0
        self.numOfFastConfirmations = 0
        self.depositStatus = ChainDepositStatus.INVALID
        self.minDepositAmt = 0
        self.withdrawStatus = ChainWithdrawStatus.INVALID
        self.minWithdrawAmt = 0
        self.withdrawPrecision = 0
        self.maxWithdrawAmt = 0.0
        self.withdrawQuotaPerDay = 0.0
        self.withdrawQuotaPerYear = 0.0
        self.withdrawQuotaTotal = 0.0
        self.withdrawFeeType = ""
        self.transactFeeWithdraw = 0.0
        self.minTransactFeeWithdraw = 0.0
        self.maxTransactFeeWithdraw = 0.0
        self.transactFeeRateWithdraw = 0.0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.chain, format_data + "Chain")
        PrintBasic.print_basic(self.baseChain, format_data + "Base Chain")
        PrintBasic.print_basic(self.baseChainProtocol, format_data + "Base Chain Protocol")
        PrintBasic.print_basic(self.numOfConfirmations, format_data + "numOfConfirmations")
        PrintBasic.print_basic(self.numOfFastConfirmations, format_data + "numOfFastConfirmations")
        PrintBasic.print_basic(self.depositStatus, format_data + "depositStatus")
        PrintBasic.print_basic(self.minDepositAmt, format_data + "minDepositAmount")
        PrintBasic.print_basic(self.withdrawStatus, format_data + "withdrawStatus")
        PrintBasic.print_basic(self.minWithdrawAmt, format_data + "minWithdrawAmount")
        PrintBasic.print_basic(self.withdrawPrecision, format_data + "withdrawPrecision")
        PrintBasic.print_basic(self.maxWithdrawAmt, format_data + "maxWithdrawAmount")
        PrintBasic.print_basic(self.withdrawQuotaPerDay, format_data + "withdrawQuotaPerDay")
        PrintBasic.print_basic(self.withdrawQuotaPerYear, format_data + "withdrawQuotaPerYear")
        PrintBasic.print_basic(self.withdrawQuotaTotal, format_data + "withdrawQuotaTotal")
        PrintBasic.print_basic(self.withdrawFeeType, format_data + "withdrawFeeType")
        PrintBasic.print_basic(self.transactFeeWithdraw, format_data + "transactFeeWithdraw")
        PrintBasic.print_basic(self.minTransactFeeWithdraw, format_data + "minTransactFeeWithdraw")
        PrintBasic.print_basic(self.maxTransactFeeWithdraw, format_data + "maxTransactFeeWithdraw")
        PrintBasic.print_basic(self.transactFeeRateWithdraw, format_data + "transactFeeRateWithdraw")



class MarketTicker:
    """
    The ticker information.

    :member
        amount: The aggregated trading volume in last 24 hours (rotating 24h).
        count: The number of completed trades of last 24 hours (rotating 24h).
        open: The opening price of a nature day (Singapore time).
        close: The last price of a nature day (Singapore time).
        low: The low price of a nature day (Singapore time).
        high: The high price of a nature day (Singapore time).
        vol: The aggregated trading value in last 24 hours (rotating 24h).
        symbol: The trading symbol of this object, e.g. btcusdt, bccbtc.
        bid: Best bid price.
        bidSize: Best bid size.
        ask: Best ask price.
        askSize: Best ask size.
    """

    def __init__(self):
        self.amount = 0.0
        self.count = 0
        self.open = 0.0
        self.close = 0.0
        self.low = 0.0
        self.high = 0.0
        self.vol = 0.0
        self.symbol = ""
        self.bid = 0.0
        self.bidSize = 0.0
        self.ask = 0.0
        self.askSize = 0.0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.amount, format_data + "Amount")
        PrintBasic.print_basic(self.count, format_data + "Count")
        PrintBasic.print_basic(self.open, format_data + "Opening Price")
        PrintBasic.print_basic(self.close, format_data + "Last Price")
        PrintBasic.print_basic(self.low, format_data + "Low Price")
        PrintBasic.print_basic(self.high, format_data + "High Price")
        PrintBasic.print_basic(self.vol, format_data + "Vol")
        PrintBasic.print_basic(self.symbol, format_data + "Trading Symbol")
        PrintBasic.print_basic(self.bid, format_data + "Best Bid Price")
        PrintBasic.print_basic(self.bidSize, format_data + "Best Bid Size")
        PrintBasic.print_basic(self.ask, format_data + "Best Ask Price")
        PrintBasic.print_basic(self.askSize, format_data + "Best Ask Size")


from huobi.client.market import MarketClient
from huobi.constant import *
from huobi.utils import *

market_client = MarketClient(init_log=True)
interval = CandlestickInterval.MIN5
symbol = "ethusdt"
list_obj = market_client.get_candlestick(symbol, interval, 10)
LogInfo.output("---- {interval} candlestick for {symbol} ----".format(interval=interval, symbol=symbol))
LogInfo.output_list(list_obj)



















from huobi.client.account import AccountClient
from huobi.constant import *

# get accounts
from huobi.utils import *

account_client = AccountClient(api_key=g_api_key,
                              secret_key=g_secret_key)
list_obj = account_client.get_account_balance_by_subuid(sub_uid=g_sub_uid)
LogInfo.output_list(list_obj)




from huobi.client.subuser import SubuserClient
from huobi.constant import *
from huobi.utils import *

subuser_client = SubuserClient(api_key=g_api_key, secret_key=g_secret_key)
sub_uid = '122946475'
access_key = '7ab679d7-b9fee8ed-9cd4cd8a-bgbfh5tv3f'

result = subuser_client.post_subuser_apikey_deletion(sub_uid, access_key)
LogInfo.output(result)



from huobi.model.market.market_detail import MarketDetail

class MarketDetailReq:
    """
     The 24H trade statistics received by request of trade statistics only once.

     :member
         rep: The topic you subscribed.
         ts: The UNIX formatted timestamp generated by server in UTC.
         trade_statistics: The trade statistics.
     """

    def __init__(self):
        self.rep = 0
        self.ts = 0
        self.data = MarketDetail()

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.ts, format_data + "Timestamp")
        PrintBasic.print_basic(self.rep, format_data + "Channel")
        self.data.print_object()


from huobi.client.margin import MarginClient
from huobi.constant import *
from huobi.utils import *

margin_client = MarginClient(api_key=g_api_key, secret_key=g_secret_key)
transfer_id = margin_client.post_repay_margin_order(loan_id=7440184, amount=100.004083)
LogInfo.output("transfer id : {id}".format(id=transfer_id))




from huobi.client.account import AccountClient
from huobi.constant import *


def callback(account_change_event: 'AccountChangeEvent'):
    account_change_event.print_object()
    print()


account_client = AccountClient(api_key=g_api_key,
                              secret_key=g_secret_key,
                              init_log=True)
# account_client.sub_account_update(AccountBalanceMode.TOTAL, callback)
account_client.sub_account_update(AccountBalanceMode.BALANCE, callback)






class TradeDetailEvent:
    """
    The trade received by subscription of trade.

    :member
        symbol: The symbol you subscribed.
        timestamp: The UNIX formatted timestamp generated by server in UTC.
        trade_list: The trade list. The content is Trade class.
    """

    def __init__(self):
        self.ch = ""
        self.id = 0
        self.ts = 0
        self.data = list()


    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.ch, format_data + "Channel")
        PrintBasic.print_basic(self.id, format_data + "ID")
        PrintBasic.print_basic(self.ts, format_data + "Unix Time")
        if len(self.data):
            for trade_detail in self.data:
                trade_detail.print_object()
                print()


from huobi.client.generic import GenericClient
from huobi.utils import *


generic_client = GenericClient()
list_obj = generic_client.get_exchange_info()
LogInfo.output("---- Supported symbols ----")
for symbol in list_obj.symbol_list:
    LogInfo.output(symbol.symbol)

LogInfo.output("---- Supported currencies ----");
for currency in list_obj.currencies:
    LogInfo.output(currency)




from huobi.client.market import MarketClient


def callback(price_depth_event: 'PriceDepthBboEvent'):
    price_depth_event.print_object()
    print()



def error(e: 'HuobiApiException'):
    print(e.error_code + e.error_message)

market_client = MarketClient()
market_client.sub_pricedepth_bbo("btcusdt", callback, error)



from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant.system import HttpMethod
from huobi.model.account import *
from huobi.utils import *


class PostAccountTransferService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/account/transfer"

        def parse(dict_data):
            data = dict_data.get("data", {})
            return default_parse(data, AccountTransferResult, [])

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)



from huobi.model.market.depth_entry import DepthEntry


class PriceDepth:
    """
    The price depth information.

    :member
        ts: The UNIX formatted timestamp in UTC.
        version:
        bids: The list of the bid depth. The content is DepthEntry class.
        asks: The list of the ask depth. The content is DepthEntry class.

    """
    def __init__(self):
        self.ts = 0
        self.version = 0
        self.bids = list()
        self.asks = list()

    @staticmethod
    def json_parse(dict_data):
        price_depth_obj = PriceDepth()
        price_depth_obj.ts = dict_data.get("ts")
        price_depth_obj.version = dict_data.get("version")
        bid_list = list()
        bids_array = dict_data.get("bids", [])
        for item in bids_array:
            depth_entry = DepthEntry.json_parse(item)
            bid_list.append(depth_entry)
        ask_list = list()
        asks_array = dict_data.get("asks", [])
        for item in asks_array:
            depth_entry = DepthEntry.json_parse(item)
            ask_list.append(depth_entry)
        price_depth_obj.bids = bid_list
        price_depth_obj.asks = ask_list

        return price_depth_obj

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.ts, format_data + "UTC Time")
        PrintBasic.print_basic(self.version, format_data + "Version")
        if len(self.bids):
            i = 0
            print(format_data, "---- Top " + str(len(self.bids)) + " bids ----")
            for entry in self.bids:
                i = i + 1
                print(format_data, str(i) + ": price: " + str(entry.price) + ", amount: " + str(entry.amount))
                #print()

        if len(self.asks):
            i = 0
            print(format_data, "---- Top " + str(len(self.asks)) + " asks ----")
            for entry in self.asks:
                i = i + 1
                print(format_data, str(i) + ": price: " + str(entry.price) + ", amount: " + str(entry.amount))
                #print()


from huobi.constant import ApiVersion


class WebsocketRequest(object):

    def __init__(self):
        self.subscription_handler = None
        self.auto_close = False   # close connection after receive data, for subscribe set False, for request set True
        self.is_trading = False
        self.is_mbp_feed = False
        self.error_handler = None
        self.json_parser = None
        self.update_callback = None
        self.api_version = ApiVersion.VERSION_V1  # v1 as default




from huobi.client.wallet import WalletClient
from huobi.constant import *
from huobi.utils import *


wallet_client = WalletClient(api_key=g_api_key, secret_key=g_secret_key)
list_obj = wallet_client.get_deposit_withdraw(op_type=DepositWithdraw.DEPOSIT, currency=None, from_id=1, size=10, direct=QueryDirection.PREV)
LogInfo.output_list(list_obj)

list_obj = wallet_client.get_deposit_withdraw(op_type=DepositWithdraw.WITHDRAW, currency=None, from_id=1, size=10, direct=QueryDirection.NEXT)
LogInfo.output_list(list_obj)






import json
from huobi.utils.time_service import get_current_timestamp
from huobi.constant import DepthStep


def kline_channel(symbol, interval):
    channel = dict()
    channel["sub"] = "market." + symbol + ".kline." + interval
    channel["id"] = str(get_current_timestamp())
    return json.dumps(channel)


def trade_detail_channel(symbol):
    channel = dict()
    channel["sub"] = "market." + symbol + ".trade.detail"
    channel["id"] = str(get_current_timestamp())
    return json.dumps(channel)


def price_depth_channel(symbol, step_type=DepthStep.STEP0):
    channel = dict()
    channel["sub"] = "market." + symbol + ".depth." + step_type
    channel["id"] = str(get_current_timestamp())
    return json.dumps(channel)


def price_depth_bbo_channel(symbol):
    channel = dict()
    channel["sub"] = "market." + symbol + ".bbo"
    channel["id"] = str(get_current_timestamp())
    return json.dumps(channel)


def orders_update_channel(symbol):
    channel = dict()
    channel["action"] = "sub"
    channel["ch"] = "orders#{symbol}".format(symbol=symbol)
    return json.dumps(channel)


def market_detail_channel(symbol):
    channel = dict()
    channel["sub"] = "market." + symbol + ".detail"
    channel["id"] = str(get_current_timestamp())
    return json.dumps(channel)


def accounts_update_channel(mode=0):
    channel = dict()
    channel["action"] = "sub"
    if mode is None:
        channel["ch"] = "accounts.update"
    else:
        channel["ch"] = "accounts.update#{mode}".format(mode=mode)
    return json.dumps(channel)


def mbp_increase_channel(symbol, levels):
    channel = dict()
    channel["sub"] = "market.{symbol}.mbp.{levels}".format(symbol=symbol, levels=levels)
    channel["id"] = str(get_current_timestamp())
    return json.dumps(channel)


def mbp_full_channel(symbol, levels):
    channel = dict()
    channel["sub"] = "market.{symbol}.mbp.refresh.{levels}".format(symbol=symbol, levels=levels)
    channel["id"] = str(get_current_timestamp())
    return json.dumps(channel)


def request_mbp_channel(symbol, levels):
    channel = dict()
    channel["req"] = "market.{symbol}.mbp.{levels}".format(symbol=symbol, levels=levels)
    channel["id"] = str(get_current_timestamp())
    return json.dumps(channel)


def trade_clearing_channel(symbol="*"):
    channel = dict()
    channel["action"] = "sub"
    channel["ch"] = "trade.clearing#" + symbol
    return json.dumps(channel)



from huobi.constant import *


class MarginAccountBalance:
    """
    The margin order information.

    :member
        id: Inner id.
        type: The account type.
        state: The account state.
        symbol: The symbol, like "btcusdt".
        fl_price: The trigger price.
        fl_type: The trigger type.
        risk_rate: The risk rate.
        list:Balance Object list
    """

    def __init__(self):
        self.id = 0
        self.type = AccountType.INVALID
        self.state = AccountState.INVALID
        self.symbol = ""
        self.fl_price = 0.0
        self.fl_type = 0.0
        self.risk_rate = 0.0
        self.list = []

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "ID")
        PrintBasic.print_basic(self.type, format_data + "Account Type")
        PrintBasic.print_basic(self.state, format_data + "Account State")
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        PrintBasic.print_basic(self.fl_price, format_data + "Trigger Price")
        PrintBasic.print_basic(self.fl_type, format_data + "Trigger Type")
        PrintBasic.print_basic(self.risk_rate, format_data + "Risk Rate")
        if self.list and len(self.list):
            for balance_obj in self.list:
                balance_obj.print_object("\t")
                print()


from huobi.client.subuser import SubuserClient
from huobi.constant import *
from huobi.utils import *

subuser_client = SubuserClient(api_key=g_api_key, secret_key=g_secret_key)
apikey_info = subuser_client.get_user_apikey_info(122946475)
LogInfo.output_list(apikey_info)



from huobi.model.market.candlestick import Candlestick
from huobi.model.market.candlestick_event import CandlestickEvent
from huobi.model.market.candlestick_req import CandlestickReq
from huobi.model.market.last_trade_bestquote import LastTradeAndBestQuote
from huobi.model.market.pricedepth import PriceDepth
from huobi.model.market.pricedepth_event import PriceDepthEvent
from huobi.model.market.pricedepth_req import PriceDepthReq
from huobi.model.market.pricedepth_bbo import PriceDepthBbo
from huobi.model.market.pricedepth_bbo_event import PriceDepthBboEvent
from huobi.model.market.market_detail_merged import MarketDetailMerged
from huobi.model.market.market_detail import MarketDetail
from huobi.model.market.market_detail_event import MarketDetailEvent
from huobi.model.market.market_detail_req import MarketDetailReq
from huobi.model.market.trade import Trade
from huobi.model.market.trade_detail import TradeDetail
from huobi.model.market.trade_detail_event import TradeDetailEvent
from huobi.model.market.trade_detail_req import TradeDetailReq
from huobi.model.market.market_ticker import MarketTicker
from huobi.model.market.depth_entry import DepthEntry
from huobi.model.market.mbp import Mbp
from huobi.model.market.mbp_increase_event import MbpIncreaseEvent
from huobi.model.market.mbp_full_event import MbpFullEvent
from huobi.model.market.mbp_req import MbpReq






from huobi.client.market import MarketClient
from huobi.model.market import *



def callback(obj_event: 'MarketDetailReq'):
    obj_event.print_object()
    print()


sub_client = MarketClient()
sub_client.req_market_detail("btcusdt", callback)



from huobi.client.trade import TradeClient
from huobi.constant import *
from huobi.utils import *

client_order_id_test = "xxxxxx"
symbol_test = "eosusdt"

trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key)

order_id = trade_client.create_order(symbol=symbol_test, account_id=g_account_id,
                                     order_type=OrderType.BUY_LIMIT,
                                     source=OrderSource.API,
                                     amount=18.0,
                                     price=0.292, client_order_id=client_order_id_test)
LogInfo.output("created order id : {id}, {client_order_id}".format(id=order_id, client_order_id=client_order_id_test))
result = trade_client.cancel_client_order(client_order_id=client_order_id_test)
LogInfo.output("cancel result {id}".format(id=result))




from huobi.client.account import AccountClient
from huobi.client.margin import MarginClient
from huobi.constant import *

# get accounts
from huobi.utils import *

account_client = AccountClient(api_key=g_api_key,
                              secret_key=g_secret_key)

LogInfo.output("========= case 1 get spot (SDK encapsulated api) =========")
account_obj = account_client.get_account_by_type_and_symbol(account_type=AccountType.SPOT, symbol=None)
account_obj.print_object()

LogInfo.output("========= case 2 get margin-eosusdt (SDK encapsulated api) =========")
account_obj = account_client.get_account_by_type_and_symbol(account_type=AccountType.MARGIN, symbol="eosusdt")
if account_obj:
    account_obj.print_object()

LogInfo.output("========= case 3 get margin-eosusdt (SDK encapsulated api) =========")
account_obj = account_client.get_account_by_type_and_symbol(account_type=AccountType.SUPER_MARGIN, symbol=None)
if account_obj:
    account_obj.print_object()

LogInfo.output("========= case 4 get margin-eosusdt (original api, supported) =========")
margin_client = MarginClient(api_key=g_api_key, secret_key=g_secret_key)
list_obj = margin_client.get_margin_account_balance(symbol="eosusdt")
LogInfo.output_list(list_obj)

LogInfo.output("========= case 5 get cross-margin (original api, supported) =========")
account_balance = margin_client.get_cross_margin_account_balance()
account_balance.print_object()


from huobi.model.subuser.subuser_creation import SubuserCreation
from huobi.model.subuser.subuser_transferability import SubuserTransferability
from huobi.model.subuser.subuser_apikey_generation import SubuserApikeyGeneration
from huobi.model.subuser.user_apikey_info import UserApikeyInfo
from huobi.model.subuser.subuser_apikey_modification import SubuserApikeyModification



from huobi.constant import *


class MarketStatus:
    """
    The Huobi market status info.

    :member
        marketStatus: .
        haltStartTime: .
        haltEndTime: .
        haltReason:
        affectedSymbols:
    """

    def __init__(self):
        self.marketStatus = MarketStatus.NORMAL
        self.haltStartTime = -1
        self.haltEndTime = -1
        self.affectedSymbols = ""



from huobi.client.trade import TradeClient
from huobi.constant import *
from huobi.utils import LogInfo

trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key)

currency_test = "trx"
trans_id_one = trade_client.transfer_between_futures_and_pro(amount=400, currency=currency_test,
                                                               transfer_type=TransferFuturesPro.TO_FUTURES)
LogInfo.output(trans_id_one)



trans_id_two = trade_client.transfer_between_futures_and_pro(amount=400, currency=currency_test,
                                                               transfer_type=TransferFuturesPro.TO_PRO)
LogInfo.output(trans_id_two)



from huobi.client.subuser import SubuserClient
from huobi.constant import *
from huobi.utils import *

subuser_client = SubuserClient(api_key=g_api_key, secret_key=g_secret_key)
sub_uids = '122946475'

transferability_result = subuser_client.post_set_subuser_transferability(sub_uids, False)
LogInfo.output_list(transferability_result)




class TransactFeeRate:
    """
    The transact fee rate.

    :member
        symbol: symbol like "btcusdt"
        makerFeeRate: maker fee rate
        takerFeeRate: taker fee rate
        actualMakerRate: actual maker fee rate
        actualTakerRate: actual taker fee rate
    """

    def __init__(self):
        self.symbol = ""
        self.makerFeeRate = ""
        self.takerFeeRate = ""
        self.actualMakerRate = ""
        self.actualTakerRate = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        PrintBasic.print_basic(self.makerFeeRate, format_data + "makerFeeRate")
        PrintBasic.print_basic(self.takerFeeRate, format_data + "takerFeeRate")
        PrintBasic.print_basic(self.actualMakerRate, format_data + "actualMakerRate")
        PrintBasic.print_basic(self.actualTakerRate, format_data + "actualTakerRate")


from huobi.constant import *
from huobi.model.market import *


class CandlestickEvent:
    """
    The candlestick/kline data received by subscription of candlestick/kline.

    :member
        ch: the topic you subscribed
        ts: the UNIX formatted timestamp generated by server in UTC.
        tick: the data of candlestick/kline.
    """

    def __init__(self):
        self.ch = ""
        self.ts = 0
        self.tick = Candlestick()


    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.ts, format_data + "Unix Time")
        PrintBasic.print_basic(self.ch, format_data + "Channel")
        if self.tick:
            self.tick.print_object()




from huobi.client.market import MarketClient
from huobi.constant import *
from huobi.exception.huobi_api_exception import HuobiApiException
from huobi.model.market.candlestick_event import CandlestickEvent


def callback(candlestick_event: 'CandlestickEvent'):
    candlestick_event.print_object()
    print("\n")


def error(e: 'HuobiApiException'):
    print(e.error_code + e.error_message)

market_client = MarketClient()
market_client.sub_candlestick("btcusdt,ethusdt", CandlestickInterval.MIN1, callback, error)




from huobi.model.margin.loan_order import LoanOrder
from huobi.model.margin.margin_account_balance import MarginAccountBalance
from huobi.model.margin.loan_ino import LoanInfo
from huobi.model.margin.margin_loan_ino import MarginLoanInfo
from huobi.model.margin.cross_margin_loan_ino import CrossMarginLoanInfo
from huobi.model.margin.cross_margin_account_balance import CrossMarginAccountBalance





from huobi.client.market import MarketClient
from huobi.constant import *
from huobi.exception.huobi_api_exception import HuobiApiException

def callback(candlestick_req: 'CandlestickReq'):
    candlestick_req.print_object()

def error(e: 'HuobiApiException'):
    print(e.error_code + e.error_message)

sub_client = MarketClient(init_log=True)
#sub_client.request_candlestick_event("btcusdt", CandlestickInterval.MIN1, callback, from_ts_second=None, end_ts_second=None, error_handler=None)
#sub_client.request_candlestick_event("btcusdt", CandlestickInterval.MIN1, callback, from_ts_second=1571124360, end_ts_second=1571129820)
#sub_client.request_candlestick_event("btcusdt", CandlestickInterval.MIN1, callback, from_ts_second=1569361140, end_ts_second=0)
#sub_client.request_candlestick_event("btcusdt", CandlestickInterval.MIN1, callback, from_ts_second=1569379980)
sub_client.req_candlestick("btcusdt", CandlestickInterval.MIN1, callback)



from huobi.client.account import AccountClient
from huobi.constant import *


# get accounts
from huobi.utils import *

account_client = AccountClient(api_key=g_api_key,
                              secret_key=g_secret_key)
list_obj = account_client.get_balance(account_id=g_account_id)
LogInfo.output_list(list_obj)


from huobi.constant import *


class GeneralRepayLoanRecord:
    """
    The general repay loan record information.

    :member
        repayId: repayment transaction ID.
        repayTime: repayment transaction time (unix time in millisecond).
        accountId: repayment account ID.
        currency: repayment currency, like "usdt".
        repaidAmount: repaid amount.
        transactIds: ID list of original loan transactions (arranged by order of repayment time).
        nextId: search the start ID in the next page (return only when there is data in the next page).
    """

    def __init__(self):
        self.repayId = None
        self.repayTime = None
        self.accountId = None
        self.currency = None
        self.repaidAmount = None
        self.transactIds = Transact()
        self.nextId = None

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.repayId, format_data + "repayId")
        PrintBasic.print_basic(self.repayTime, format_data + "repayTime")
        PrintBasic.print_basic(self.accountId, format_data + "accountId")
        PrintBasic.print_basic(self.currency, format_data + "currency")
        PrintBasic.print_basic(self.repaidAmount, format_data + "repaidAmount")
        PrintBasic.print_basic(self.transactIds, format_data + "transactIds")
        PrintBasic.print_basic(self.nextId, format_data + "nextId")

        print()


class Transact:

    """
    The general repay loan record information.

    :member
        transactId: original loan transaction ID.
        repaidPrincipal: principal repaid.
        repaidInterest: interest repaid.
        paidHt: HT paid.
        paidPoint: point paid.
    """

    def __init__(self):
        self.transactId = None
        self.repaidPrincipal = None
        self.repaidInterest = None
        self.paidHt = None
        self.paidPoint = None




from huobi.constant import *


class LoanOrder:
    """
    The margin order information.

    :member
        id: The order id.
        user_id: The user id.
        account_type: The account type which created the loan order.
        currency: The currency name.
        loan_amount: The amount of the origin loan.
        loan_balance: The amount of the loan left.
        interest_rate: The loan interest rate.
        interest_amount: The accumulated loan interest.
        interest_balance: The amount of loan interest left.
        state: The loan stats, possible values: created, accrual, cleared, invalid.
        created_at: The UNIX formatted timestamp in UTC when the order was created.
        accrued_at: The UNIX formatted timestamp in UTC when the last accrue happened.
    """

    def __init__(self):
        self.currency = ""
        self.deduct_rate = 0
        self.paid_point = 0.0
        self.deduct_currency = ""
        self.user_id = 0
        self.created_at = 0
        self.account_id = 0
        self.paid_coin = 0.0
        self.loan_amount = 0.0
        self.interest_amount = 0.0
        self.deduct_amount = 0.0
        self.loan_balance = 0.0
        self.interest_balance = 0.0
        self.updated_at = 0
        self.accrued_at = 0
        self.interest_rate = 0.0
        self.id = 0
        self.state = LoanOrderState.INVALID

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.deduct_rate, format_data + "Deduct Rate")
        PrintBasic.print_basic(self.paid_point, format_data + "Paid Point")
        PrintBasic.print_basic(self.deduct_currency, format_data + "Deduct Currency")
        PrintBasic.print_basic(self.user_id, format_data + "User Id")
        PrintBasic.print_basic(self.created_at, format_data + "Create Time")
        PrintBasic.print_basic(self.account_id, format_data + "Account Id")
        PrintBasic.print_basic(self.paid_coin, format_data + "Paid Coin")
        PrintBasic.print_basic(self.loan_amount, format_data + "Load Amount")
        PrintBasic.print_basic(self.interest_amount, format_data + "Interest Amount")
        PrintBasic.print_basic(self.deduct_amount, format_data + "Deduct Amount")
        PrintBasic.print_basic(self.loan_balance, format_data + "Loan Balance")
        PrintBasic.print_basic(self.interest_balance, format_data + "Interest Balance")
        PrintBasic.print_basic(self.updated_at, format_data + "Update Time")
        PrintBasic.print_basic(self.accrued_at, format_data + "Accrued Time")
        PrintBasic.print_basic(self.interest_rate, format_data + "Interest Rate")
        PrintBasic.print_basic(self.id, format_data + "ID")
        PrintBasic.print_basic(self.state, format_data + "Loan Order State")



from huobi.client.margin import MarginClient
from huobi.constant import *
from huobi.utils import LogInfo

margin_client = MarginClient(api_key=g_api_key, secret_key=g_secret_key)

list_result = margin_client.get_general_repayment_loan_records(limit=10)
LogInfo.output_list(list_result)







from huobi.constant import *
from huobi.model.trade import *
from huobi.utils.input_checker import *


class TradeClient(object):

    def __init__(self, **kwargs):
        """
        Create the request client instance.
        :param kwargs: The option of request connection.
            api_key: The public key applied from Huobi.
            secret_key: The private key applied from Huobi.
            url: The URL name like "https://api.huobi.pro".
            init_log: to init logger
        """
        self.__kwargs = kwargs

    def get_feerate(self, symbols: 'str') -> list:
        """
        Get the candlestick/kline for the specified symbol. The data number is 150 as default.

        :param symbol: The symbol, like "btcusdt". To query hb10, put "hb10" at here. (mandatory)
        :param interval: The candlestick/kline interval, MIN1, MIN5, DAY1 etc. (mandatory)
        :param size: The start time of of requested candlestick/kline data. (optional)
        :param start_time: The start time of of requested candlestick/kline data. (optional)
        :param end_time: The end time of of requested candlestick/kline data. (optional)
        :return: The list of candlestick/kline data.
        """
        check_symbol(symbols)

        params = {
            "symbols": symbols
        }

        from huobi.service.trade.get_feerate import GetFeeRateService
        return GetFeeRateService(params).request(**self.__kwargs)

    def get_transact_feerate(self, symbols: 'str') -> list:
        """
        The request of get transact fee rate list.

        :param symbols: The symbol, like "btcusdt,htusdt". (mandatory)
        :return: The transact fee rate list.
        """
        check_symbol(symbols)

        params = {
            "symbols": symbols
        }

        from huobi.service.trade.get_transact_feerate import GetTransactFeeRateService
        return GetTransactFeeRateService(params).request(**self.__kwargs)

    def sub_order_update(self, symbols: 'str', callback, error_handler=None):
        """
        Subscribe order changing event. If a order is created, canceled etc, server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(order_update_event: 'OrderUpdateEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass
        :return:  No return
        """
        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        check_should_not_none(callback, "callback")

        params = {
            "symbol_list" : symbol_list,
        }

        from huobi.service.trade.sub_order_update_v2 import SubOrderUpdateV2Service
        SubOrderUpdateV2Service(params).subscribe(callback, error_handler, **self.__kwargs)

    def req_order_list(self, symbol: 'str', account_id: int, callback, order_states:'str',
                       order_types:'str'=None, start_date:'str'=None, end_date:'str'=None, from_id=None,
                       direct=None, size=None, client_req_id:'str'=None, error_handler=None):
        """
        request order list.

        :param symbol: The symbol, like "btcusdt".
        :param order_states: order status, can be one state or many state sepearted by comma, such as "submitted,partial-filled,partial-canceled,filled,canceled,created"
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(candlestick_event: 'CandlestickEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass
        :return: No return
        """
        check_should_not_none(symbol, "symbol")
        check_should_not_none(order_states, "states")
        check_should_not_none(account_id, "account-d")
        check_should_not_none(callback, "callback")
        params = {
            "symbol": symbol,
            "account-id" : account_id,
            "states" : order_states,
            "types" : order_types,
            "start-date" : start_date,
            "end-date" : end_date,
            "from": from_id,
            "direct" : direct,
            "size" : size,
            "client-req-id" : client_req_id
        }

        from huobi.service.trade.req_order_list import ReqOrderListService
        ReqOrderListService(params).subscribe(callback, error_handler, **self.__kwargs)

    def req_order_detail(self, order_id: 'str', callback,
                                client_req_id:'str'=None, error_handler=None):
        """
        Subscribe candlestick/kline event. If the candlestick/kline is updated, server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param interval: The candlestick/kline interval, MIN1, MIN5, DAY1 etc.
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(candlestick_event: 'CandlestickEvent'):
                        pass
        :param client_req_id: client request ID
        :param error_handler: The error handler will be called if subscription failed or error happen between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass
        :return: No return
        """
        check_should_not_none(order_id, "order_id")
        check_should_not_none(callback, "callback")
        params = {
            "order-id": order_id,
            "cid": client_req_id,
        }

        from huobi.service.trade.req_order_detail import ReqOrderDetailService
        ReqOrderDetailService(params).subscribe(callback, error_handler, **self.__kwargs)

    def get_order(self, order_id: 'int') -> Order:
        """
        Get the details of an order.

        :param order_id: The order id. (mandatory)
        :return: The information of order.
        """
        check_should_not_none(order_id, "order_id")

        params = {
            "order_id": order_id,
        }

        from huobi.service.trade.get_order_by_id import GetOrderByIdService
        return GetOrderByIdService(params).request(**self.__kwargs)

    def get_order_by_client_order_id(self, client_order_id):
        check_should_not_none(client_order_id, "clientOrderId")

        params = {
            "clientOrderId": client_order_id,
        }

        from huobi.service.trade.get_order_by_client_order_id import GetOrderByClientOrderIdService
        return GetOrderByClientOrderIdService(params).request(**self.__kwargs)

    def get_orders(self, symbol: 'str', order_state: 'OrderState', order_type: 'OrderType' = None,
                              start_date: 'str' = None, end_date: 'str' = None, start_id: 'int' = None,
                              size: 'int' = None, direct=None) -> list:
        check_symbol(symbol)
        check_should_not_none(order_state, "order_state")
        start_date = format_date(start_date, "start_date")
        end_date = format_date(end_date, "end_date")

        params = {
            "symbol" : symbol,
            "types" : order_type,
            "start-date" : start_date,
            "end-date" : end_date,
            "from" : start_id,
            "states" : order_state,
            "size" :  size,
            "direct" : direct
        }

        from huobi.service.trade.get_orders import GetOrdersService
        return GetOrdersService(params).request(**self.__kwargs)

    def get_open_orders(self, symbol: 'str', account_id: 'int', side: 'OrderSide' = None,
                        size: 'int' = None, from_id=None, direct=None) -> list:
        """
        The request of get open orders.

        :param symbol: The symbol, like "btcusdt". (mandatory)
        :param account_id: account id (mandatory)
        :param side: The order side, buy or sell. If no side defined, will return all open orders of the account. (optional)
        :param size: The number of orders to return. Range is [1, 500]. (optional)
        :param direct: 1:prev  order by ID asc from from_id, 2:next order by ID desc from from_id
        :param from_id: start ID for search
        :return: The orders information.
        """
        check_symbol(symbol)
        check_range(size, 1, 500, "size")
        check_should_not_none(account_id, "account_id")
        params = {
            "symbol" : symbol,
            "account-id" : account_id,
            "side" : side,
            "size" : size,
            "from" : from_id,
            "direct" : direct
        }

        from huobi.service.trade.get_open_orders import GetOpenOrdersService
        return GetOpenOrdersService(params).request(**self.__kwargs)

    def get_history_orders(self, symbol=None, start_time=None, end_time=None, size=None, direct=None)-> list:
        """
        Transfer Asset between Futures and Contract.

        :param direct:
        :param symbol: The target sub account uid to transfer to or from. (optional)
        :param start_time: The crypto currency to transfer. (optional)
        :param end_time: The amount of asset to transfer. (optional)
        :param size: The type of transfer, need be "futures-to-pro" or "pro-to-futures" (optional)
        :return: The Order list.
        """
        params = {
            "symbol" :symbol,
            "start-time" : start_time,
            "end-time" : end_time,
            "size" : size,
            "direct" : direct
        }

        from huobi.service.trade.get_history_orders import GetHistoryOrdersService
        return GetHistoryOrdersService(params).request(**self.__kwargs)

    def get_match_result(self, symbol: 'str', order_type: 'OrderSide' = None, start_date: 'str' = None,
                         end_date: 'str' = None,
                         size: 'int' = None,
                         from_id: 'int' = None,
                         direct:'str'=None):
        """
        Search for the trade records of an account.

        :param symbol: The symbol, like "btcusdt" (mandatory).
        :param order_type: The types of order to include in the search (optional).
        :param start_date: Search starts date in format yyyy-mm-dd. (optional).
        :param end_date: Search ends date in format yyyy-mm-dd. (optional).
        :param size: The number of orders to return, range [1-100]. (optional).
        :param from_id: Search order id to begin with. (optional).
        :return:
        """

        check_symbol(symbol)
        start_date = format_date(start_date, "start_date")
        end_date = format_date(end_date, "end_date")
        check_range(size, 1, 100, "size")

        params = {
            "symbol" : symbol,
            "start-date" : start_date,
            "end-date" : end_date,
            "types" : order_type,
            "size" : size,
            "from" : from_id,
            "direct" : direct
        }

        from huobi.service.trade.get_match_results import GetMatchResultsService
        return GetMatchResultsService(params).request(**self.__kwargs)

    def get_match_results_by_order_id(self, order_id: 'int') -> list:
        """
        Get detail match results of an order.

        :param order_id: The order id. (mandatory)
        :return: The list of match result.
        """
        check_should_not_none(order_id, "order_id")

        params = {
            "order_id": order_id
        }

        from huobi.service.trade.get_match_results_by_order_id import GetMatchResultsByOrderIdService
        return GetMatchResultsByOrderIdService(params).request(**self.__kwargs)

    def order_source_desc(self, account_type):
        default_source = "api"
        if account_type:
            if account_type == AccountType.MARGIN:
                return "margin-api"
        return default_source

    def create_order_param_check(self, symbol: 'str', account_id: 'int', order_type: 'OrderType', amount: 'float',
                     price: 'float', source:'str', client_order_id=None, stop_price=None, operator=None):
        check_symbol(symbol)
        check_should_not_none(account_id, "account_id")
        check_should_not_none(order_type, "order_type")
        check_should_not_none(amount, "amount")
        check_should_not_none(source, "source")

        if order_type == OrderType.SELL_LIMIT \
                or order_type == OrderType.BUY_LIMIT \
                or order_type == OrderType.BUY_LIMIT_MAKER \
                or order_type == OrderType.SELL_LIMIT_MAKER:
            check_should_not_none(price, "price")
        if order_type in [OrderType.SELL_MARKET, OrderType.BUY_MARKET]:
            price = None

        params = {
            "account-id" : account_id,
            "amount" : amount,
            "price": price,
            "symbol": symbol,
            "type": order_type,
            "source": source,
            "client-order-id": client_order_id,
            "stop-price": stop_price,
            "operator": operator
        }

        return params

    def create_order(self, symbol: 'str', account_id: 'int', order_type: 'OrderType', amount: 'float',
                     price: 'float', source:'str', client_order_id=None, stop_price=None, operator=None) -> int:
        """
        Make an order in huobi.

        :param symbol: The symbol, like "btcusdt". (mandatory)
        :param account_id: Account id. (mandatory)
        :param order_type: The order type. (mandatory)
        :param source: The order source. (mandatory)
                for spot, it's "api", see OrderSource.API
                for margin, it's "margin-api", see OrderSource.MARGIN_API
                for super margin, it's "super-margin-api", see OrderSource.SUPER_MARGIN_API
        :param amount: The amount to buy (quote currency) or to sell (base currency). (mandatory)
        :param price: The limit price of limit order, only needed for limit order. (mandatory for buy-limit, sell-limit, buy-limit-maker and sell-limit-maker)
        :param client_order_id: unique Id which is user defined and must be unique in recent 24 hours
        :param stop_price: Price for auto sell to get the max benefit
        :param operator: the condition for stop_price, value can be "gte" or "lte",  gte – greater than and equal (>=), lte – less than and equal (<=)
        :return: The order id.
        """

        params = self.create_order_param_check(symbol, account_id, order_type, amount,
                     price, source, client_order_id, stop_price, operator)
        from huobi.service.trade.post_create_order import PostCreateOrderService
        return PostCreateOrderService(params).request(**self.__kwargs)

    def create_spot_order(self, symbol: 'str', account_id: 'int', order_type: 'OrderType', amount: 'float',
                       price: 'float', client_order_id=None, stop_price=None,
                       operator=None) -> int:
        order_source = OrderSource.API
        return self.create_order(symbol=symbol, account_id=account_id, order_type=order_type, amount=amount,
                       price=price, source=order_source, client_order_id=client_order_id, stop_price=stop_price,
                       operator=operator)

    def create_margin_order(self, symbol: 'str', account_id: 'int', order_type: 'OrderType', amount: 'float',
                       price: 'float', client_order_id=None, stop_price=None,
                       operator=None) -> int:
        order_source = OrderSource.MARGIN_API
        return self.create_order(symbol=symbol, account_id=account_id, order_type=order_type, amount=amount,
                                 price=price, source=order_source, client_order_id=client_order_id,
                                 stop_price=stop_price,
                                 operator=operator)

    def create_super_margin_order(self, symbol: 'str', account_id: 'int', order_type: 'OrderType', amount: 'float',
                            price: 'float', client_order_id=None, stop_price=None,
                            operator=None) -> int:
        order_source = OrderSource.SUPER_MARGIN_API
        return self.create_order(symbol=symbol, account_id=account_id, order_type=order_type, amount=amount,
                                 price=price, source=order_source, client_order_id=client_order_id,
                                 stop_price=stop_price,
                                 operator=operator)

    def cancel_order(self, symbol, order_id):
        check_symbol(symbol)
        check_should_not_none(order_id, "order_id")

        params = {
            "order_id" : order_id
        }

        from huobi.service.trade.post_cancel_order import PostCancelOrderService
        return PostCancelOrderService(params).request(**self.__kwargs)

    def cancel_orders(self, symbol, order_id_list)->BatchCancelResult:
        """
        Submit cancel request for cancelling multiple orders.

        :param symbol: The symbol, like "btcusdt". (mandatory)
        :param order_id_list: The list of order id. the max size is 50. (mandatory)
        :return: No return
        """
        check_symbol(symbol)
        check_should_not_none(order_id_list, "order_id_list")
        check_list(order_id_list, 1, 50, "order_id_list")

        string_list = list()
        for order_id in order_id_list:
            string_list.append(str(order_id))

        params = {
            "order-ids" : string_list
        }

        from huobi.service.trade.post_batch_cancel_order import PostBatchCancelOrderService
        return PostBatchCancelOrderService(params).request(**self.__kwargs)

    def cancel_open_orders(self, account_id, symbols: 'str'=None , side=None, size=None)->BatchCancelCount:
        """
        Request to cancel open orders.

        :param symbols: The symbol, like "btcusdt".
        :param account_type: Account type. (mandatory)
        :param side: The order side, buy or sell. If no side defined, will cancel all open orders of the account. (optional)
        :param size: The number of orders to cancel. Range is [1, 100]. (optional)
        :return: Status of batch cancel result.
        """
        check_should_not_none(account_id, "account_id")

        params = {
            "account-id": account_id,
            "symbol" : symbols,
            "side" : side,
            "size" : size
        }

        from huobi.service.trade.post_batch_cancel_open_order import PostBatchCancelOpenOrderService
        return PostBatchCancelOpenOrderService(params).request(**self.__kwargs)

    def cancel_client_order(self, client_order_id)->int:
        """
        Request to cancel open orders.

        :param client_order_id: user defined unique order id
        """
        check_should_not_none(client_order_id, "client-order-id")

        params = {
            "client-order-id" : client_order_id
        }

        from huobi.service.trade.post_cancel_client_order import PostCancelClientOrderService
        return PostCancelClientOrderService(params).request(**self.__kwargs)

    def transfer_between_futures_and_pro(self, currency: 'str', amount: 'float',
                                        transfer_type: 'TransferFuturesPro')-> int:
        """
        Transfer Asset between Futures and Contract.

        :param sub_uid: The target sub account uid to transfer to or from. (mandatory)
        :param currency: The crypto currency to transfer. (mandatory)
        :param amount: The amount of asset to transfer. (mandatory)
        :param transfer_type: The type of transfer, need be "futures-to-pro" or "pro-to-futures" (mandatory)
        :return: The order id.
        """
        check_currency(currency)
        check_should_not_none(currency, "currency")
        check_should_not_none(amount, "amount")
        check_should_not_none(transfer_type, "transfer_type")
        params = {
            "currency" : currency,
            "amount" : amount,
            "type" : transfer_type

        }

        from huobi.service.trade.post_transfer_futures_pro import PostTransferFuturesProService
        return PostTransferFuturesProService(params).request(**self.__kwargs)

    def batch_create_order(self, order_config_list) -> int:
        """
        Make an order in huobi.
        :param order_config_list: order config list, it can batch create orders, and each order config check as below
            : items as below
                :param symbol: The symbol, like "btcusdt". (mandatory)
                :param account_type: Account type. (mandatory)
                :param order_type: The order type. (mandatory)
                :param amount: The amount to buy (quote currency) or to sell (base currency). (mandatory)
                :param price: The limit price of limit order, only needed for limit order. (mandatory for buy-limit, sell-limit, buy-limit-maker and sell-limit-maker)
                :param client_order_id: unique Id which is user defined and must be unique in recent 24 hours
                :param stop_price: Price for auto sell to get the max benefit
                :param operator: the condition for stop_price, value can be "gte" or "lte",  gte – greater than and equal (>=), lte – less than and equal (<=)
        :return: The order id.
        """

        check_should_not_none(order_config_list, "order_config_list")
        check_list(order_config_list, 1, 10, "create order config list")

        new_config_list = list()
        for item in order_config_list:
            new_item = self.create_order_param_check(
                item.get("symbol", None),
                item.get("account_id", None),
                item.get("order_type", None),
                item.get("amount", None),
                item.get("price", None),
                item.get("source", None),
                item.get("client_order_id", None),
                item.get("stop-price", None),
                item.get("operator", None))

            new_config_list.append(new_item)

        from huobi.service.trade.post_batch_create_order import PostBatchCreateOrderService
        return PostBatchCreateOrderService(new_config_list).request(**self.__kwargs)

    def sub_trade_clearing(self, symbols: 'str', callback, error_handler=None):
        """
        Subscribe trade clearing by symbol

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
                        "*" for all symbols
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(price_depth_event: 'PriceDepthEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass

        :return:  No return
        """
        check_should_not_none(symbols, "symbols")
        symbol_list = symbols.split(",")
        if ("*" in symbol_list):
            symbol_list = ["*"]
        else:
            check_symbol_list(symbol_list)

        check_should_not_none(callback, "callback")

        params = {
            "symbol_list": symbol_list,
        }

        from huobi.service.trade.sub_trade_clearing_v2 import SubTradeClearingV2Service
        SubTradeClearingV2Service(params).subscribe(callback, error_handler, **self.__kwargs)




from huobi.client.trade import TradeClient
from huobi.constant import *


def callback(obj_event: 'TradeClearingEvent'):
    obj_event.print_object()


trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key, init_log=True)
trade_client.sub_trade_clearing("eosusdt", callback)



from huobi.client.account import AccountClient
from huobi.constant import *

account_client = AccountClient(api_key=g_api_key,
                               secret_key=g_secret_key)

account_type = "spot"
asset_valuation = account_client.get_account_asset_valuation(account_type=account_type, valuation_currency="usd")
asset_valuation.print_object()




class FeeRate:
    """
    The account information for spot account, margin account etc.

    :member
        symbol: The symbol, like "btcusdt".
        maker_fee: maker fee rate
        taker_fee: taker fee rate

    """

    def __init__(self):
        self.symbol = ""
        self.maker_fee = ""
        self.taker_fee = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        PrintBasic.print_basic(self.maker_fee, format_data + "Maker Fee")
        PrintBasic.print_basic(self.taker_fee, format_data + "Taker Fee")




class EtfSwapInOut:
    """
    :member
    """

    def __init__(self):
        self.code = 0
        self.data = None
        self.message = ""
        self.success = False

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.code, format_data + "Return Code")
        PrintBasic.print_basic_bool(self.data, format_data + "Data")
        PrintBasic.print_basic(self.message, format_data + "Message")
        PrintBasic.print_basic_bool(self.success, format_data + "Success")



class SubuserApikeyGeneration:
    """
    The trade information with price and amount etc.

    :member
        accessKey:
        secretKey:
        note:
        permission: "trade,readOnly",
        ipAddresses":
    """

    def __init__(self):
        self.accessKey = ""
        self.secretKey = ""
        self.note = ""
        self.permission = ""
        self.ipAddresses = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.accessKey, format_data + "accessKey")
        PrintBasic.print_basic(self.secretKey, format_data + "secretKey")
        PrintBasic.print_basic(self.note, format_data + "note")
        PrintBasic.print_basic(self.permission, format_data + "permission")
        PrintBasic.print_basic(self.ipAddresses, format_data + "ipAddresses")



import time

from huobi.connection.websocket_req_client import *
from huobi.utils.channels_request import *
from huobi.model.market import *


class ReqPriceDepthService:
    def __init__(self, params):
        self.params = params

    def subscribe(self, callback, error_handler, **kwargs):
        symbol_list = self.params["symbol_list"]
        step = self.params["step"]

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(request_price_depth_channel(symbol, step))
                time.sleep(0.01)

        def parse(dict_data):
            price_depth_event = PriceDepthReq()
            price_depth_event.id = dict_data.get("id")
            price_depth_event.rep = dict_data.get("rep")
            data = dict_data.get("data", {})
            price_depth_obj = PriceDepth.json_parse(data)
            price_depth_event.data = price_depth_obj
            return price_depth_event

        WebSocketReqClient(**kwargs).execute_subscribe_v1(subscription,
                                            parse,
                                            callback,
                                            error_handler)






from huobi.client.subuser import SubuserClient
from huobi.constant import *
from huobi.utils import *
import string
import random

subuser_client = SubuserClient(api_key=g_api_key, secret_key=g_secret_key)
userName = ''.join(random.choices(string.ascii_uppercase + string.digits, k=7))
params = {"userList": [
    {
        "userName": userName,
        "note": "huobi"
    }
]}

userList = subuser_client.post_create_subuser(params)
LogInfo.output_list(userList)



import time
from huobi.client.trade import TradeClient
from huobi.constant import *
from huobi.utils import *


trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key)

client_order_id_header = str(int(time.time()))

symbol_eosusdt = "eosusdt"

client_order_id_eos_01 = client_order_id_header + symbol_eosusdt + "01"
client_order_id_eos_02 = client_order_id_header + symbol_eosusdt + "02"
client_order_id_eos_03 = client_order_id_header + symbol_eosusdt + "03"

buy_limit_eos_01 = {
    "account_id":g_account_id,
    "symbol":symbol_eosusdt,
    "order_type":OrderType.BUY_LIMIT,
    "source":OrderSource.API,
    "amount":50,
    "price": 0.12,
    "client_order_id" : client_order_id_eos_01
 }

buy_limit_eos_02 = {
    "account_id":g_account_id,
    "symbol":symbol_eosusdt,
    "order_type":OrderType.BUY_LIMIT,
    "source": OrderSource.API,
    "amount":7,
    "price": 0.80,
    "client_order_id" : client_order_id_eos_02
 }

buy_limit_eos_03 = {
    "account_id":g_account_id,
    "symbol":symbol_eosusdt,
    "order_type":OrderType.BUY_LIMIT,
    "source": OrderSource.API,
    "amount":20,
    "price": 0.252,
    "client_order_id" : client_order_id_eos_03
 }

order_config_list = [
    buy_limit_eos_01,
    buy_limit_eos_02,
    buy_limit_eos_03
]

create_result = trade_client.batch_create_order(order_config_list=order_config_list)
LogInfo.output_list(create_result)

order_id_list = []
if create_result and len(create_result):
    for item in create_result:
        order_id_list.append(item.order_id)

    result = trade_client.cancel_orders(symbol_eosusdt, order_id_list)
    result.print_object()







from huobi.constant import *

class CandlestickReq:
    """
    The candlestick/kline data received by subscription of candlestick/kline.

    :member
        rep: the Channel or topic you subscribed.
        id: the UNIX formatted timestamp generated by server in UTC.
        data: the data of candlestick/kline.

    """

    def __init__(self):
        self.rep = ""
        self.id = 0
        self.data = list()

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.rep, format_data + "Channel")
        PrintBasic.print_basic(self.id, format_data + "Unix Time")
        print()
        if len(self.data):
            for row in self.data:
                row.print_object()
                print()


from huobi.model.trade.batch_cancel_result import BatchCancelResult
from huobi.model.trade.batch_cancel_count import BatchCancelCount
from huobi.model.trade.feerate import FeeRate
from huobi.model.trade.matchresult import MatchResult
from huobi.model.trade.order import Order
from huobi.model.trade.order_detail_req import OrderDetailReq
from huobi.model.trade.order_list_item import OrderListItem
from huobi.model.trade.order_list_req import OrderListReq
from huobi.model.trade.order_update_event import OrderUpdateEvent
from huobi.model.trade.order_update import OrderUpdate
from huobi.model.trade.batch_create_order import BatchCreateOrder
from huobi.model.trade.transact_feerate import TransactFeeRate
from huobi.model.trade.trade_clearing import TradeClearing
from huobi.model.trade.trade_clearing_event import TradeClearingEvent






from huobi.constant import *

class ReferenceCurrency:
    """
    The Huobi supported static reference information for each currency.

    :member
        currency: currency
        instStatus: Instrument status
        chains: chain list
    """

    def __init__(self):
        self.currency = ""
        self.instStatus = InstrumentStatus.INVALID
        self.chains = []


    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.instStatus, format_data + "Instrument Status")
        if self.chains and len(self.chains):
            for chain_obj in self.chains:
                chain_obj.print_object("\t")
                print()



from huobi.client.margin import MarginClient
from huobi.constant import *
from huobi.utils import *

margin_client = MarginClient(api_key=g_api_key, secret_key=g_secret_key)
list_obj = margin_client.get_margin_loan_info(symbols="btcusdt,ethusdt,eosusdt")
LogInfo.output_list(list_obj)




class AccountPointTransferResult:
    """
    The account change information received by subscription of account.

    :member
        timestamp: The UNIX formatted timestamp generated by server in UTC.
        change_type: The event that asset change notification related.
        account_change_list: The list of account change, the content is AccountChange class

    """

    def __init__(self):
        self.transactId = ""
        self.transactTime = 0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.transactId, format_data + "transactId")
        PrintBasic.print_basic(self.transactTime, format_data + "transactTime")
        self.data.print_object()



class UnitPrice:

    def __init__(self):
        self.currency = ""
        self.amount = 0.0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.amount, format_data + "Amount")


from huobi.client.account import AccountClient
from huobi.constant import *


def callback(account_balance_req: 'AccountBalanceReq'):
    account_balance_req.print_object()


account_client = AccountClient(api_key=g_api_key, secret_key=g_secret_key)
account_client.req_account_balance(callback=callback, client_req_id=None)



from huobi.client.algo import AlgoClient
from huobi.constant import *
from huobi.utils import *

symbol_test = "adausdt"
account_id = g_account_id

algo_client = AlgoClient(api_key=g_api_key, secret_key=g_secret_key)
order_id = algo_client.create_order(symbol=symbol_test, account_id=account_id, order_side=OrderSide.BUY,
                                    order_type=AlgoOrderType.LIMIT, order_size=65, order_price=0.08, stop_price=0.085,
                                    client_order_id="test004")
LogInfo.output("created order id : {id}".format(id=order_id))



from huobi.constant import *
from huobi.utils.json_parser import fill_obj, default_parse_list_dict


class TradeClearing:
    """
    The detail order information.

    :member
        order_id: The order id.
        symbol: The symbol, like "btcusdt".
        tradePrice: trade price.
        tradeVolume: trade Volume.
        orderSide: order Side, more to see OrderSide
        orderType: order type, more to see OrderType
        aggressor: is aggressor, only true or false
        tradeId: trade ID.
        tradeTime: trade Time.
        transactFee: transact Fee.
        feeDeduct: Deduct Fee.
        feeDeductType: fee Deduct Type, current only support ht and point
    """

    def __init__(self):
        self.symbol = ""
        self.orderId = 0
        self.tradePrice = ""
        self.tradeVolume = ""
        self.orderSide = OrderSide.INVALID
        self.orderType = OrderType.INVALID
        self.aggressor = False
        self.tradeId = 0
        self.tradeTime = 0
        self.transactFee = ""
        self.feeDeduct = ""
        self.feeDeductType = FeeDeductType.INVALID

    @staticmethod
    def json_parse(json_data):
        if json_data.get("orderId", None):
            return default_parse_list_dict(json_data, TradeClearing)
        else:
            return TradeClearing()

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.orderId, format_data + "Order Id")
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        PrintBasic.print_basic(self.tradePrice, format_data + "Trade Price")
        PrintBasic.print_basic(self.tradeVolume, format_data + "Trade Volume")
        PrintBasic.print_basic(self.orderSide, format_data + "Order Side")
        PrintBasic.print_basic(self.orderType, format_data + "Order Type")
        PrintBasic.print_basic(self.aggressor, format_data + "is Taker")
        PrintBasic.print_basic(self.tradeId, format_data + "Trade Id")
        PrintBasic.print_basic(self.tradeTime, format_data + "Trade Time")
        PrintBasic.print_basic(self.transactFee, format_data + "Transact Fee")
        PrintBasic.print_basic(self.feeDeduct, format_data + "Fee Deduct")
        PrintBasic.print_basic(self.feeDeductType, format_data + "Fee Deduct Type")





from huobi.client.margin import MarginClient
from huobi.constant import *
from huobi.utils import *

margin_client = MarginClient(api_key=g_api_key, secret_key=g_secret_key)

# to check not clearing loan orders
list_obj = margin_client.get_cross_margin_loan_orders(currency="usdt", state=LoanOrderState.ACCRUAL)
LogInfo.output_list(list_obj)

if list_obj and len(list_obj):
    for loan_order in list_obj:
        repay_amount = float(loan_order.loan_balance) + float(loan_order.interest_balance)
        LogInfo.output("repay loan order {id} and amount {amount}".format(id=loan_order.id, amount=repay_amount))
        order_id = margin_client.post_cross_margin_loan_order_repay(order_id=loan_order.id, amount=repay_amount)

# to check not clearing loan orders
list_obj = margin_client.get_cross_margin_loan_orders(currency="usdt", state=LoanOrderState.ACCRUAL)
LogInfo.output_list(list_obj)


from huobi.model.market import Mbp, PriceDepth


class MbpReq:
    """
    The market price depth.

    :member
        rep: request Topic
        id: The UNIX formatted timestamp generated by server in UTC.
        data: The price depth.
    """

    def __init__(self):
        self.rep = ""
        self.id = ""

        self.data = Mbp()


    @staticmethod
    def json_parse(data_json):
        mbp_event = MbpReq()
        mbp_event.id = data_json.get("id")
        mbp_event.rep = data_json.get("rep")
        data = data_json.get("data", {})
        mbp = Mbp.json_parse(data)
        mbp_event.data = mbp
        return mbp_event

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.rep, format_data + "Topic")
        PrintBasic.print_basic(self.id, format_data + "Timestamp")

        self.data.print_object(format_data + "\t")



class AccountAssetValuationResult:
    """
    The account information for spot account, margin account etc.

    :member
        balance: balance valuation bases on given valuation currency.
        timestamp: unix timestamp from server.

    """

    def __init__(self):
        self.balance = ""
        self.timestamp = 0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.balance, format_data + "balance")
        PrintBasic.print_basic(self.timestamp, format_data + "timestamp")



from huobi.client.account import AccountClient
from huobi.constant import *


# get accounts
from huobi.utils import *

api_key ='88fbec25-de8461a1-ntmuw4rrsr-bb051'
api_secret ='bde897ae-beb99d2d-cfa29656-1a0a6'



account_client = AccountClient(api_key=api_key,
                               secret_key=api_secret)
# list_obj = account_client.get_accounts()
# if list_obj and len(list_obj):
#     for account_obj in list_obj:
#         list_obj = account_client.get_balance(account_id=account_obj.id)
#         LogInfo.output("===== {account_id} , {account_type} =====".format(account_id=account_obj.id, account_type=account_obj.type))
#         if len(list_obj):
#             for obj in list_obj:
#                 if float(obj.balance) > 0.1:  # only show account with balance
#                     obj.print_object()
#                     print()

LogInfo.output("====== (SDK encapsulated api) not recommend for low performance and frequence limitation ======")
account_balance_list = account_client.get_account_balance()
if account_balance_list and len(account_balance_list):
    for account_balance_obj in account_balance_list:
        if account_balance_obj and len(account_balance_obj.list):
            PrintBasic.print_basic(account_balance_obj.id, "ID")
            PrintBasic.print_basic(account_balance_obj.type, "Account Type")
            PrintBasic.print_basic(account_balance_obj.state, "Account State")
            PrintBasic.print_basic(account_balance_obj.subtype, "Subtype")
            for balance_obj in account_balance_obj.list:
                if float(balance_obj.balance) > 0.1:  # only show account with balance
                    balance_obj.print_object("\t")
                    print()
        print()

raise Exception('mozer foker')


from huobi.constant import *
from huobi.model.account import Balance
from huobi.utils import *


class CrossMarginAccountBalance:

    """
    The account information for spot account, margin account etc.

    :member
        id: The unique account id.
        type: The type of this account, possible value: spot, margin, otc, point.
        state: The account state, possible value: working, lock.
        list: The balance list of the specified currency. The content is Balance class
    """


    def __init__(self):
        self.id = 0
        self.type = AccountType.INVALID
        self.state = AccountState.INVALID
        self.risk_rate = 0
        self.acct_balance_sum = 0.0
        self.debt_balance_sum = 0.0
        self.list = list()

    @staticmethod
    def json_parse(data_json):
        balance_list_json = data_json.get("list", [])
        data_json.pop("list")

        account_balance = default_parse_list_dict(data_json, CrossMarginAccountBalance)
        account_balance.list = default_parse_list_dict(balance_list_json, Balance, [])

        return account_balance

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "Account ID")
        PrintBasic.print_basic(self.type, format_data + "Account Type")
        PrintBasic.print_basic(self.state, format_data + "Account State")
        PrintBasic.print_basic(self.risk_rate, format_data + "Risk Rate")
        PrintBasic.print_basic(self.acct_balance_sum, format_data + "Total Balance")
        PrintBasic.print_basic(self.debt_balance_sum, format_data + "Debt Balance")
        if self.list and len(self.list):
            for balance in self.list:
                balance.print_object("\t")
                print()



from huobi.client.subuser import SubuserClient
from huobi.constant import *

subuser_client = SubuserClient(api_key=g_api_key, secret_key=g_secret_key)

subUid = 122946475
access_key = "abc"
note = "test"
permission = 'readOnly,trade'

result = subuser_client.post_subuser_apikey_modification(subUid, access_key, permission=permission, note=note)
result.print_object()



from huobi.constant import *


class MatchResult:
    """
    The match result information.

    :member
        created_timestamp: The UNIX formatted timestamp in UTC when the match and fill is done.
        filled_amount: The amount which has been filled.
        filled_fees: The transaction fee paid so far.
        id: The internal id.
        match_id: The match id of this match.
        order_id: The order id of this order.
        price: The limit price of limit order.
        source: The source where the order was triggered, possible values: sys, web, api, app.
        symbol: The symbol, like "btcusdt".
        type: The order type, possible values are: buy-market, sell-market, buy-limit, sell-limit,
            buy-ioc, sell-ioc, buy-limit-maker, sell-limit-maker, buy-limit-fok, sell-limit-fok, buy-stop-limit-fok, sell-stop-limit-fok.
        filled_points: deduct points
        fee_deduct_currency: deduct type, it means deduct from HT/ HT points / or other currency
        fee_currency:
    """

    def __init__(self):
        self.created_at = 0
        self.filled_amount = 0.0
        self.filled_fees = 0.0
        self.id = 0
        self.match_id = 0
        self.order_id = 0
        self.price = 0.0
        self.source = OrderSource.INVALID
        self.symbol = ""
        self.type = OrderType.INVALID
        self.role = ""
        self.filled_points = ""
        self.fee_deduct_currency = ""
        self.fee_currency = ""
        self.fee_deduct_state = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "ID")
        PrintBasic.print_basic(self.created_at, format_data + "Create Time")
        PrintBasic.print_basic(self.filled_amount, format_data + "Fill Amount")
        PrintBasic.print_basic(self.filled_fees, format_data + "Fill Fee")
        PrintBasic.print_basic(self.filled_points, format_data + "Fill Points")
        PrintBasic.print_basic(self.match_id, format_data + "Match ID")
        PrintBasic.print_basic(self.order_id, format_data + "Order ID")
        PrintBasic.print_basic(self.price, format_data + "Price")
        PrintBasic.print_basic(self.source, format_data + "Source")
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        PrintBasic.print_basic(self.type, format_data + "Order Type")
        PrintBasic.print_basic(self.role, format_data + "Role")
        PrintBasic.print_basic(self.fee_deduct_currency, format_data + "Fee Deduct Currency")
        PrintBasic.print_basic(self.fee_currency, format_data + "Fee Currency")
        PrintBasic.print_basic(self.fee_deduct_state, format_data + "Fee Deduct State")



from huobi.client.trade import TradeClient
from huobi.constant import *
from huobi.utils import *

order_id = 25943298415466
trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key)
orderObj = trade_client.get_order(order_id=order_id)
LogInfo.output("======= get order by order id : {order_id} =======".format(order_id=order_id))
orderObj.print_object()




from huobi.client.account import AccountClient
from huobi.constant import *

# get accounts
from huobi.utils import *

account_client = AccountClient(api_key=g_api_key,
                               secret_key=g_secret_key)

account_history = account_client.get_account_history(account_id=g_account_id, start_time=1600827872000, size=4)
LogInfo.output_list(account_history["data"])
LogInfo.output('Next Id: %s' % (account_history["next_id"]))



class AccountLedger:
    """
    The account ledger information.

    :member
        accountId: Account ID.
        currency: Cryptocurrency.
        transactAmt: Transaction amount (income positive, expenditure negative).
        transactType: Transaction type.
        transferType: Transfer type (only valid for transactType=transfer).
        transactId: Transaction ID.
        transactTime: Transaction time.
        transferer: Transferer’s account ID.
        transferee: Transferee’s account ID.
    """

    def __init__(self):
        self.accountId = 0
        self.currency = ""
        self.transactAmt = 0.0
        self.transactType = ""
        self.transferType = ""
        self.transactId = 0
        self.transactTime = 0
        self.transferer = 0
        self.transferee = 0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.accountId, format_data + "Account ID")
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.transactAmt, format_data + "Transaction Amount")
        PrintBasic.print_basic(self.transactType, format_data + "Transaction Type")
        PrintBasic.print_basic(self.transferType, format_data + "Transfer Type")
        PrintBasic.print_basic(self.transactId, format_data + "Transaction ID")
        PrintBasic.print_basic(self.transactTime, format_data + "Transaction Time")
        PrintBasic.print_basic(self.transferer, format_data + "Transferer’s Account ID")
        PrintBasic.print_basic(self.transferee, format_data + "Transferee’s Account ID")


from huobi.connection.impl.restapi_request import RestApiRequest
from huobi.constant import *
from huobi.model.generic import *


class GenericClient(object):

    def __init__(self, **kwargs):
        """
        Create the request client instance.
        :param kwargs: The option of request connection.
            api_key: The public key applied from Huobi.
            secret_key: The private key applied from Huobi.
            url: The URL name like "https://api.huobi.pro".
            init_log: to init logger
        """
        self.__kwargs = kwargs

    def get_exchange_timestamp(self) -> int:
        """
        Get the timestamp from Huobi server. The timestamp is the Unix timestamp in millisecond.
        The count shows how many milliseconds passed from Jan 1st 1970, 00:00:00.000 at UTC.
        e.g. 1546300800000 is Thu, 1st Jan 2019 00:00:00.000 UTC.

        :return: The timestamp in UTC
        """

        params = {}

        from huobi.service.generic.get_exchange_timestamp import GetExchangeTimestampService
        return GetExchangeTimestampService(params).request(**self.__kwargs)

    def get_exchange_currencies(self) -> list():
        """
        Get all the trading assets and currencies supported in huobi.
        The information of trading instrument, including base currency, quote precision, etc.

        :return: The information of trading currencies.
        """

        params = {}

        from huobi.service.generic.get_exchange_currencies import GetExchangeCurrenciesService
        return GetExchangeCurrenciesService(params).request(**self.__kwargs)

    def get_exchange_symbols(self) -> list():
        """
        Get all the trading assets and currencies supported in huobi.
        The information of trading instrument etc.

        :return: The information of trading instrument.
        """

        params = {}

        from huobi.service.generic.get_exchange_symbols import GetExchangeSymbolsService
        return GetExchangeSymbolsService(params).request(**self.__kwargs)

    def get_exchange_info(self) -> ExchangeInfo:
        """
        Get all the trading assets and currencies supported in huobi.
        The information of trading instrument, including base currency, quote precision, etc.

        :return: The information of trading instrument and currencies.
        """

        ret = ExchangeInfo()
        ret.symbol_list = self.get_exchange_symbols()
        ret.currencies = self.get_exchange_currencies()
        return ret

    def get_reference_currencies(self, currency: 'str' = None, is_authorized_user: 'bool' = None) -> list:
        """
        Get all the trading assets and currencies supported in huobi.
        The information of trading instrument, including base currency, quote precision, etc.

        :param currency: btc, ltc, bch, eth, etc ...(available currencies in Huobi Global)
        :param is_authorized_user: is Authorized user? True or False
        :return: The information of trading instrument and currencies.
        """

        params = {
            "currency": currency,
            "authorizedUser": is_authorized_user
        }

        from huobi.service.generic.get_reference_currencies import GetReferenceCurrenciesService
        return GetReferenceCurrenciesService(params).request(**self.__kwargs)

    def get_system_status(self) -> str:
        """
        get system status

        :return: system status.
        """

        from huobi.service.generic.get_system_status import GetSystemStatusService
        return GetSystemStatusService({}).request(**self.__kwargs)

    def get_market_status(self):
        from huobi.service.generic.get_market_status import GetMarketStatusService
        return GetMarketStatusService({}).request(**self.__kwargs)



import time

from huobi.utils import *

from huobi.connection.websocket_req_client import *
from huobi.model.market import *
from huobi.utils.channels_request import *


class ReqCandleStickService:
    def __init__(self, params):
        self.params = params

    def subscribe(self, callback, error_handler, **kwargs):
        symbol_list = self.params["symbol_list"]
        interval = self.params["interval"]
        from_ts_second = self.params.get("from_ts_second", None)
        end_ts_second = self.params.get("end_ts_second", None)

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(request_kline_channel(symbol, interval, from_ts_second, end_ts_second))
                time.sleep(0.01)

        def parse(dict_data):
            return default_parse(dict_data, CandlestickReq, Candlestick)

        WebSocketReqClient(**kwargs).execute_subscribe_v1(subscription,
                                            parse,
                                            callback,
                                            error_handler)






from huobi.model.etf.etf_swap_config import EtfSwapConfig
from huobi.model.etf.etf_swap_list import EtfSwapList
from huobi.model.etf.etf_swap_in_out import EtfSwapInOut
from huobi.model.etf.unitprice import UnitPrice



class PriceDepthBbo:
    """
    The price depth information.

    :member
        timestamp: The UNIX formatted timestamp in UTC.
        bid: the first bid near trade value.
        bidSize: the bid size.
        ask: The first ask near trade value.
        askSize: the ask size.
        quoteTime : quote time
        symbol : trade symbol


    """
    def __init__(self):
        self.seqId = 0
        self.ask = 0.0
        self.askSize = 0.0
        self.bid = 0.0
        self.bidSize = 0.0
        self.quoteTime = 0
        self.symbol = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.seqId, format_data + "Seq ID")
        PrintBasic.print_basic(self.ask, format_data + "Ask")
        PrintBasic.print_basic(self.askSize, format_data + "Ask Size")
        PrintBasic.print_basic(self.bid, format_data + "Bid")
        PrintBasic.print_basic(self.bidSize, format_data + "Bid Size")
        PrintBasic.print_basic(self.quoteTime, format_data + "Quote Time")
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")


from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant import *
from huobi.utils.json_parser import default_parse_data_as_long


class PostCancelOrderService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        order_id = self.params["order_id"]

        def get_channel():
            path = "/v1/order/orders/{}/submitcancel"
            return path.format(order_id)

        def parse(dict_data):
            return default_parse_data_as_long(dict_data, None)
        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, get_channel(), self.params, parse)









import threading
import logging
from apscheduler.schedulers.blocking import BlockingScheduler

from huobi.connection.impl.private_def import *
from huobi.utils.time_service import get_current_timestamp


def watch_dog_job(*args):
    watch_dog_obj = args[0]

    for idx, websocket_manage in enumerate(watch_dog_obj.websocket_manage_list):
        if websocket_manage.request.auto_close:  # setting auto close no need reconnect
            pass
        elif websocket_manage.state == ConnectionState.CONNECTED:
            if watch_dog_obj.is_auto_connect:
                ts = get_current_timestamp() - websocket_manage.last_receive_time
                if ts > watch_dog_obj.heart_beat_limit_ms:
                    watch_dog_obj.logger.warning("[Sub][" + str(websocket_manage.id) + "] No response from server")
                    websocket_manage.close_and_wait_reconnect(watch_dog_obj.wait_reconnect_millisecond())
        elif websocket_manage.state == ConnectionState.WAIT_RECONNECT:
            watch_dog_obj.logger.warning("[Sub] call re_connect")
            websocket_manage.re_connect()
            pass
        elif websocket_manage.state == ConnectionState.CLOSED_ON_ERROR:
            if watch_dog_obj.is_auto_connect:
                websocket_manage.close_and_wait_reconnect(watch_dog_obj.reconnect_after_ms)
                pass


class WebSocketWatchDog(threading.Thread):
    mutex = threading.Lock()
    websocket_manage_list = list()

    def __init__(self, is_auto_connect=True, heart_beat_limit_ms=CONNECT_HEART_BEAT_LIMIT_MS, reconnect_after_ms=RECONNECT_AFTER_TIME_MS):
        threading.Thread.__init__(self)
        self.is_auto_connect = is_auto_connect
        self.heart_beat_limit_ms = heart_beat_limit_ms
        self.reconnect_after_ms = reconnect_after_ms if reconnect_after_ms > heart_beat_limit_ms else heart_beat_limit_ms
        self.logger = logging.getLogger("huobi-client")
        self.scheduler = BlockingScheduler()
        self.scheduler.add_job(watch_dog_job, "interval", max_instances=10, seconds=1, args=[self])
        self.start()

    def run(self):
        self.scheduler.start()

    def on_connection_created(self, websocket_manage):
        self.mutex.acquire()
        self.websocket_manage_list.append(websocket_manage)
        self.mutex.release()

    def on_connection_closed(self, websocket_manage):
        self.mutex.acquire()
        self.websocket_manage_list.remove(websocket_manage)
        self.mutex.release()

    # calculate next reconnect time
    def wait_reconnect_millisecond(self):
        wait_millisecond = int(self.reconnect_after_ms - self.heart_beat_limit_ms)
        now_ms = get_current_timestamp()
        wait_millisecond = wait_millisecond if wait_millisecond else 1000
        # job loop after 1 second
        return (wait_millisecond + now_ms)






from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant.system import HttpMethod
from huobi.model.generic import *
from huobi.utils import *



class GetReferenceCurrenciesService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/reference/currencies"

        def parse(dict_data):
            ret_list = []
            data_list = dict_data.get("data", [])
            if data_list and len(data_list):
                for reference_currency in data_list:
                    reference_currency_obj = default_parse(reference_currency, ReferenceCurrency, Chain)
                    ret_list.append(reference_currency_obj)
            return ret_list

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET, channel, self.params, parse)









from huobi.client.algo import AlgoClient
from huobi.constant import *

account_id = g_account_id
client_order_id = "test002"

# get specific order by clientOrderId
algo_client = AlgoClient(api_key=g_api_key, secret_key=g_secret_key)
result = algo_client.get_order(client_order_id)
result.print_object()



from huobi.client.generic import GenericClient
from huobi.utils import *


generic_client = GenericClient()
list_obj = generic_client.get_exchange_symbols()
if len(list_obj):
    for idx, row in enumerate(list_obj):
        LogInfo.output("------- number " + str(idx) + " -------")
        row.print_object()






from huobi.client.trade import TradeClient
from huobi.constant import *
from huobi.utils import *

symbol = "htusdt"
trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key)
list_obj = trade_client.get_orders(symbol=symbol, order_state=OrderState.FILLED,
                                    order_type=OrderType.BUY_LIMIT, start_date=None, end_date=None,
                                   start_id=None, size=None, direct=QueryDirection.PREV)

LogInfo.output("===== step 1 ==== {symbol} {count} orders found".format(symbol=symbol, count=len(list_obj)))
LogInfo.output_list(list_obj)

symbol = "eosusdt"
list_obj = trade_client.get_orders(symbol=symbol, order_state=OrderState.CANCELED,
                                    order_type=OrderType.BUY_LIMIT, start_date="2020-05-21", end_date=None,
                                   start_id=None, size=None, direct=QueryDirection.PREV)
LogInfo.output("===== step 2 ==== {symbol} {count} canceled buy limit orders found".format(symbol=symbol, count=len(list_obj)))
LogInfo.output_list(list_obj)

list_obj = trade_client.get_orders(symbol=symbol, order_state=OrderState.FILLED,
                                    order_type=None, start_date=None, end_date=None,
                                   start_id=None, size=None, direct=QueryDirection.PREV)
LogInfo.output("===== step 3 ==== {symbol} {count} filled orders found".format(symbol=symbol, count=len(list_obj)))
LogInfo.output_list(list_obj)



from huobi.client.market import MarketClient


def callback(trade_event: 'TradeDetailEvent'):
    print("---- trade_event:  ----")
    trade_event.print_object()
    print()



market_client = MarketClient(init_log=True)
market_client.sub_trade_detail("btcusdt,eosusdt", callback)

